/* ---------------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	foid.h
 *
 * Date:	September 1994
 * 
 * Purpose:	Interface for the convertion of Friendly-named Object 
 *			Identifiers.
 *
 *			This interface converts a string, containing a mixed
 *			OID ( e.g. 1.3.dod.1 ) into 2 strings.
 *			- One containing:  iso.org.dod.internet
 *			- One containing:  1.3.6.1
 *			The first is called Friendly-name, the second normal
 *			OID. If wanted, a description can also be fetched.
 *			
 *			The input-string can also contain names of sub-
 *			identifiers without specifying the way to the root.
 *			For example, "dod.mgmt" will be converted to:
 *				iso.org.dod.internet.mgmt
 *				1.3.6.1.2
 *
 *		The working of this module depends entirely on the databases.
 *		Be sure that the databases are located in the 
 *		$(SNMP_PARTYMIB_INITPATH) directory, and that they have been
 *		made by Rap and Crap;
 *
 * Auther(s):	Erwin Schrijver
 *
 * Remarks:	When you edit this file, please rebuild foid.o !!
 *
 * ---------------------------------------------------------------------------
 */

#ifndef FOID_H
#define FOID_H

#include <snmpTypes.h>


/* Constants */
#define O_NAMELEN	40	/* Max. length of object-name      	    */
#define LINELEN		128	/* Max. length of line in Database 	    */
#define ARG_LEN		512	/* Max. length of OID in Friendly-name mode */

/* Values of nError ( foidConv() )					    */
#define	NBUF		-1	/* Buffer not large enough		    */
#define NTOTAL		-2	/* Convertion not completed entirely	    */
#define ILLEGAL		-3	/* Illegal OID				    */

/* Symbols            DO NOT EDIT !!!!  (depends on Rap)                    */
#define DOTDOT	".."		/* Unallowed in OID			    */ 
#define DOT	"."		/* Separator in OID			    */
#define EOL	"\0"		/* End Of Line				    */
#define ROOT	"ROOT"		/* Top of MIB-tree			    */
#define UNKNOWN	"UNKNOWN"	/* Default name of branch		    */
#define COMMENT	"--"		/* Comment in database files		    */
#define TREE	"/foid.tree"	/* Filename of database with tree-structure */
#define DESC	"/foid.desc"	/* Filename of database with descriptions   */
#define DATA_PATH "SNMP_PARTYMIB_INITPATH"

typedef	struct	FoidT	{	/* Structure of database		    */	
		char	szBranch[O_NAMELEN];
		char	szNode[O_NAMELEN];
		AsnInt	nBranch;
		AsnInt	nDesIndex;
} FoidT;



/* ---------------------------------------------------------------------------
 * An example of using foid:
 *
 * main
 * {
 *	FoidT	*pData;
 *	AsnOcts	octFriendly;
 *	AsnOcts	octOID;
 *	AsnInt	nDescription;
 *	AsnInt	nError;
 *	...
 *
 *	foidOpen( &pData );
 *
 *	octFriendly.octs = malloc( 256 );
 *	octFriendly.octetLen = 256;
 *	octOID.octs = malloc( 256 );
 *	octOID.octetLen = 256;
 *	octDesc.octs = malloc( 512 );
 *	octDesc.octetLen = 512;
 *
 *	...
 *
 *	foidConv( pData, "1.3.dod.1", octFriendly, octOID, 
 *		  &nDescription, &nError );
 *	printf( "%s\n%s\n", octFriendly.octs, octOID.octs );
 *
 *	foidDescription( nDescription, octDesc );
 *	printf( "%s\n", octDesc.octs );
 *
 *	...
 *
 *	foidConv( pData, "iso.3.6", octFriendly, octOID, 
 *		  &nDescription, &nError );
 *	printf( "%s\n%s\n", octFriendly.octs, octOID.octs );
 *
 *	...
 *
 *	free( octFriendly.octs );
 *	free( octOID.octs );
 *	free( octDesc.octs );
 *
 *	foidClose( &pData );
 * }
 *
 * ---------------------------------------------------------------------------
 */

/* ---------------------------------------------------------------------------
 *
 * Function:	foidOpen
 *
 * Arguments:	pData, address of pointer to FoidT to contain database (OUT)
 *
 * Return:	OK on success, ERROR when datafile couldn't be opened,
 *		or when datafile is incorrect, or when not enough memory
 *		could be allocated.
 *		pData points to the database on success. 	
 *
 * Purpose:	This function reads the entire database in memory.
 *
 * Remarks:	This function should be called before any convertion is made.
 *		After all convertions, before the end of the program, the
 *		used memory has to be released by calling foidClose().
 *		Be aware that you'll have to use pData both for foidConv()
 *		and foidClose(), so don't loose this pointer!
 *
 * ---------------------------------------------------------------------------
 */

EXTERN STATUS foidOpen( FoidT **pData );






/* ---------------------------------------------------------------------------
 *
 * Function:	foidClose
 *
 * Arguments:	pData,		address of pointer to database (IN/OUT).
 *
 * Return:	STATUS, OK on success, FAILED when database was already closed.	
 *
 * Purpose:	This function releases the memory allocated by foidOpen().
 *
 * Remarks:	On success, pData is made NULL.
 *
 * ---------------------------------------------------------------------------
 */

EXTERN STATUS foidClose( FoidT **pData );






/* ---------------------------------------------------------------------------
 *
 * Function:	foidConv
 *
 * Arguments:	pData,		pointer to database (IN).
 *		szIn,		string with mixed OID (IN).
 *		octFriendly,	Octet-string with will be filled with 
 *					friendly name (OUT).
 *		octOID,		Octet-string with will be filled with OID (OUT).
 *		nDesIndex,	address of Indexnumber to description. This
 *				address will contain zero if no convertion is 
 *				made ( due to an error or fault ), and -1 when
 *				the object has no description (OUT).
 *		nError,		status of the convertion (OUT):
 *			ILLEGAL illegal OID,
 *			NBUF	Buffers (octFriendly and/or octOID) not large
 *					enough.
 *			NTOTAL  Conversion not completed entirely.  
 *
 * Return:	STATUS, OK on success, FAILED when OID isn't correct or when
 *		OID could not be found, or when octOID/octFriendly is/are
 *		too small.
 *		ERROR when not enough memory could be allocated.	
 *
 * Purpose:	This function converts a mixed OID.
 *
 * Remarks:	The value of (*nDesIndex) is the number you'll have to use
 *		with foidDescription() to obtain the description of the
 *		specified object. 
 *
 * ---------------------------------------------------------------------------
 */
EXTERN STATUS foidConv( FoidT *pData, char *szIn, AsnOcts octFriendly, 
			AsnOcts octOID, AsnInt *nDesIndex, AsnInt *nError );






/* ---------------------------------------------------------------------------
 *
 * Function:	foidDescription
 *
 * Arguments:	nIndex,		index of description (IN).
 *		octBuf,		Buffer to contain description (OUT).
 *
 * Return:	STATUS, OK on succes, ERROR when database couldn't be opened
 *		or when database is incorrect.
 *		FAILED when desciption isn`t in database, or when octBuf is
 *		too small. 	
 *
 * Purpose:	This function gives the discription refered to by nIndex.
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------------
 */
EXTERN STATUS foidDescription( AsnInt nIndex, AsnOcts octBuf );

#endif		/* conditional include */
