/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    APIb.c (basic)
 *
 * Date:        September 1994
 *
 * Purpose:     This file implements the UT-SNMP api.
 *
 * Author(s):	Antoinne Olle'man, Eric van Hengstum & Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */

#include <sim.h>
#include <apiMSG.h>

/*
 *  module variables
 */
static AsnOct address[128] ;
static AsnOct *spmAddress="spm" ;
static AsnInt handle ;



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbOpen
 *
 * Arguments:   name, names the local entity (IN)
 *				role, the role which the entity intends to performs (IN)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function starts and initializes this entity. Firstly,
 *				the UT-SPM is activated, an associating is set up, and 
 *				an introduction message from UT-APPL. to UT-SPM.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbOpen( 
	AsnOcts *name, AsnInt role )
{
    pid_t pid;
	AsnInt type=UTB_OPEN_TYPE;
    char cmd[256];
	static AsnInt init = 0 ;

    if( ! init ) {
    	InitNibbleMem( (6*1024), 1024 );
		if( msgOpen( &handle ) != OK ) return( ERROR ) ;
		init = 1 ;
	}

    memcpy( address, name->octs, name->octetLen );
    if( simOpen( address, spmAddress ) != OK ) {
        simClose( spmAddress );
        return( ERROR );
    }
    msgPut( &handle, NULL, &type, name, &role, NULL, NULL, NULL, NULL, 
		    NULL, NULL, NULL );
    if( simSend( spmAddress, handle ) != OK ) {
		simClose( spmAddress );
        return( ERROR );
    }
    return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbClose
 *
 * Arguments:   none
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function closes the entity. First an close IPC_MSG
 *				is send to the UT-SPM, and afterwards the association is
 *				closed.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbClose( )
{
	AsnInt type=UTB_CLOSE_TYPE;

    msgPut( &handle, NULL, &type, NULL, NULL, NULL, NULL, NULL, NULL,
		    NULL, NULL, NULL );
    if( simSend( spmAddress, handle ) !=  OK ) return( ERROR );
    msgClose( handle );
    simClose( spmAddress );
    ShutdownNibbleMem( );

    return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbLook
 *
 * Arguments:   type, references the received messagetype (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function receives the inter-process communication 
 *				messages from the UT-SPM, and returns the messagetype to 
 *				the user.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbLook( AsnInt *type )
{
	AsnOct a[128] ;

    if( simReceive( a, handle ) != OK ) return( ERROR ) ;
    if( msgGet( &handle, NULL, type, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL, NULL, NULL ) != OK ) return( ERROR ) ;

    return( OK ) ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSend
 *
 * Arguments:   none
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     Generic send function.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utSend( AsnInt *id, AsnInt *type, AsnOcts *ta, AsnInt *qos,
		AsnInt *etype, AsnInt *eindex, AsnOid *lb, AsnOid *ub, 
		AsnOcts *cle, AsnOid *clt, AsnVbl *vars ) 
{
    msgPut( &handle, id, type, ta, qos, etype, eindex, lb, ub, 
        	cle, clt, vars ); 
    if( simSend( spmAddress, handle ) !=  OK ) return ERROR ;

    return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbRecv
 *
 * Arguments:   ...
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     Generic receive function.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utRecv( AsnInt *id, AsnInt *type, AsnOcts *ta, AsnInt *qos,
		AsnInt *etype, AsnInt *eindex, AsnOid *lb, AsnOid *ub, 
		AsnOcts *le, AsnOid *lt, AsnVbl *vars ) 
{
	AsnInt tp;

    if( msgGet( &handle, id, &tp, ta, qos, etype, eindex, lb, ub, 
			    le, lt, vars ) != OK ) return( ERROR ) ; 
	
	return( OK ) ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbPerror
 *
 * Arguments:   id, error number (IN/OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the print error primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbPerror( 	
	AsnInt *etype )
{
	AsnOct *utbPerrorTable[]={
		"noError", "tooBig", "noSuchName", "badValue", "readOnly", "genErr",
        "noAccess", "wrongType", "wrongLength", "wrongEncoding", 
		"wrongValue","noCreation","inconsistentValue","resourceUnavailable",
        "commitFailed", "undoFailed", "authorizationError", "notWritable",
        "inconsistentname", 
		"unKnown", "providerError", "timeOut", "unKnownContextLocalEntity",
		"noUniqueDestination", "NoSuchContextLocal", "noUndoerror", 
		"CommitFailed", "unKnownHost", "noSuchQos", "MibRegistrationFailed",
		"MibUnRegistrationFailed", "InformRegistrationFailed", 
		"InformUnRegistrationFailed", "TrapRegistrationFailed",
		"TrapUnRegistrationFailed", "IllegalOperation"
	};
    AsnInt type=UTB_PERROR_TYPE;

	if( *etype == UTB_PERROR_TYPE ) {
        utRecv( NULL, &type, NULL, NULL, etype, NULL, NULL, NULL, NULL,
                NULL, NULL );
	}
	fprintf( stderr, "%s", utbPerrorTable[*etype] );
	return OK ;
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbMibRegRq
 *
 * Arguments:   id, subtree identifier (IN)
 *				cle, contextlocalentity (IN)
 *				clt, contextlocaltime (IN)
 *				lb, subtree lower-bound (IN)
 *				ub, subtree upper-bound (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function registers an subtree at the UT-SPM.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbMibRegRq( 
	AsnInt id,
	AsnOcts *cle, AsnOid *clt, AsnOid *lb, AsnOid *ub )
{
	AsnInt type=UTB_MIBREGRQ_TYPE;
	return
		utSend( &id, &type, NULL, NULL, NULL, NULL, lb, ub, cle, clt, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbMibRegCf
 *
 * Arguments:   id, subtree identifier (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function confirms the registration of an subtree at 
 *				the UT-SPM.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbMibRegCf(
	AsnInt *id )
{
	AsnInt type=UTB_MIBREGCF_TYPE;
	return 
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
				NULL, NULL ); 
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbMibUnRegRq
 *
 * Arguments:   id, subtree identifier (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function unregisters an subtree at the UT-SPM.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbMibUnRegRq( 
	AsnInt id )
{
	AsnInt type=UTB_MIBUNREGRQ_TYPE;
	return
		utSend( &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbMibUnRegCf
 *
 * Arguments:   id, subtree identifier (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function confirms the unregistration of an subtree at 
 *				the UT-SPM.
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbMibUnRegCf( 
	AsnInt *id )
{
	AsnInt type=UTB_MIBUNREGCF_TYPE;
	return 
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
				NULL, NULL ); 
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetRq
 *
 * Arguments:   id, message identifier (IN)
 *				ta, remote entity (IN)
 *				qos, quality of service (IN) 
 * 				cle, contextlocalentity (IN)
 *				clt, contextlocaltime (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Get-Request service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetRq( 
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_GETRQ_TYPE;
	return
		utSend( &id, &type, ta, &qos, NULL, NULL, NULL, NULL, cle, clt,
				vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetIn
 *
 * Arguments:   id, message identifier (OUT)
 * 				cle, contextlocalentity (OUT)
 *				clt, contextlocaltime (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Get-Indication service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetIn( 
	AsnInt *id, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_GETIN_TYPE;
	return 
		utRecv(	id, &type, NULL, NULL, NULL, NULL, NULL, NULL,
        		cle, clt, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetRs
 *
 * Arguments:   id, message identifier (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Get-Response service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetRs( 	
	AsnInt id, AsnInt etype, AsnInt eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_GETRS_TYPE;
	return
        utSend( &id, &type, NULL, NULL, &etype, &eindex, NULL, NULL, NULL,
                NULL, NULL );

}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetCf
 *
 * Arguments:   id, message identifier (OUT)
 * 				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Get-confirmation service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_GETCF_TYPE;
	return
		utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL, 
				NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetnextRq
 *
 * Arguments:   id, message identifier (IN)
 *				ta, remote entity (IN)
 *				qos, quality of service (IN) 
 * 				cle, contextlocalentity (IN)
 *				clt, contextlocaltime (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getnext-Request service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetnextRq(	
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_GETNEXTRQ_TYPE;
	return
		utSend( &id, &type, ta, &qos, NULL, NULL, NULL, NULL, cle, clt,
				vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetnextIn
 *
 * Arguments:   id, message identifier (OUT)
 * 				cle, contextlocalentity (OUT)
 *				clt, contextlocaltime (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getnext-Indication service 
 *				primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetnextIn(	
	AsnInt *id, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_GETNEXTIN_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, cle, clt, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetnextRs
 *
 * Arguments:   id, message identifier (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getnext-Response service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetnextRs(	
	AsnInt id, AsnInt etype, AsnInt eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_GETNEXTRS_TYPE;
	return
        utSend( &id, &type, NULL, NULL, &etype, &eindex, NULL, NULL, NULL,
                NULL, NULL );

}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetnextCF
 *
 * Arguments:   id, message identifier (OUT)
 * 				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getnext-Confirmation service 
 *				primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetnextCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_GETBULKCF_TYPE;
	return
		utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL,
				NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetbulkRq
 *
 * Arguments:   id, message identifier (IN)
 *				ta, remote entity (IN)
 *				qos, quality of service (IN) 
 *				nonr, non-repeaters (IN)
 *				maxr, max-repetitions (IN)
 * 				cle, contextlocalentity (IN)
 *				clt, contextlocaltime (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getbulk-Request service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetbulkRq(	
	AsnInt id, AsnOcts *ta, AsnInt qos, AsnInt nonr, AsnInt maxr,
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_GETBULKRQ_TYPE;
	return
		utSend( &id, &type, ta, &qos, &nonr, &maxr, NULL, NULL, cle, clt,
				vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetbulkIn
 *
 * Arguments:   id, message identifier (OUT)
 *				maxr, max-repetitions (OUT)
 * 				cle, contextlocalentity (OUT)
 *				clt, contextlocaltime (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getbulk-Indication service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetbulkIn( 
	AsnInt *id, AsnInt *maxr, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_GETBULKIN_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL, maxr, NULL, NULL, cle, clt, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetbulkRs
 *
 * Arguments:   id, message identifier (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getbulk-Response service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetbulkRs( 
	AsnInt id, AsnInt etype, AsnInt eindex,
	AsnVbl *vars )
{
	AsnInt type=UTB_GETBULKRS_TYPE;
	return
        utSend( &id, &type, NULL, NULL, &etype, &eindex, NULL, NULL, NULL,
                NULL, NULL );

}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbGetIn
 *
 * Arguments:   id, subtree identifier (OUT)
 * 				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Getbulk-Confirmation service 
 *				primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbGetbulkCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_GETBULKCF_TYPE;
	return
		utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, 
				NULL, NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetRq
 *
 * Arguments:   id, message identifier (IN)
 *				ta, remote entity (IN)
 *				qos, quality of service (IN) 
 * 				cle, contextlocalentity (IN)
 *				clt, contextlocaltime (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Set-Request service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetRq(
	AsnInt id, AsnOcts *ta, AsnInt qos,
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_SETRQ_TYPE;
	return
		utSend( &id, &type, ta, &qos, NULL, NULL, NULL, NULL, cle, clt,
				vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetIn
 *
 * Arguments:   id, message identifier (OUT)
 * 				cle, contextlocalentity (OUT)
 *				clt, contextlocaltime (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Set-Indication service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetIn(
	AsnInt *id, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_SETIN_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL , NULL, NULL, NULL, cle, clt, 
				vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetRs
 *
 * Arguments:   id, subtree identifier (IN)
 * 				eindex, errorindex (IN)
 *				etype, errorstatus (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Set-response service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetRs( 
	AsnInt id, AsnInt etype, AsnInt eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_SETRS_TYPE;
	return
		utSend( &id, &type, NULL, NULL, &etype, &eindex, NULL, NULL, NULL,
				NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetUndoIn
 *
 * Arguments:   id, message identifier (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the ... service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetUndoIn( 
	AsnInt *id )
{
	AsnInt type=UTB_SETUNDOIN_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetCommitIn
 *
 * Arguments:   id, message identifier (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the ... service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetCommitIn( 
	AsnInt *id )
{
	AsnInt type=UTB_SETCOMMITIN_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL ) ;
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetUndoRs
 *
 * Arguments:   id, subtree identifier (IN)
 * 				etype, errorstatus (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the ... service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetUndoRs( 
	AsnInt id, AsnInt etype )
{
	AsnInt type=UTB_SETUNDORS_TYPE;
	return
		utSend( &id, &type, NULL, NULL, &etype, NULL, NULL, NULL, NULL, 
				NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetCommitRs
 *
 * Arguments:   id, subtree identifier (IN)
 *				etype, errorstatus (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the ... service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetCommitRs( 
	AsnInt id, AsnInt etype )
{
	AsnInt type=UTB_SETCOMMITRS_TYPE;
	return
		utSend( &id, &type, NULL, NULL, &etype, NULL, NULL, NULL, NULL, 
				NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetRollForwards
 *
 * Arguments:   id, message identifier (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the ... service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetRollForwards(
	AsnInt *id )
{
	AsnInt type=UTB_SETROLLFORWARDS_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
				 NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetRollBackwards
 *
 * Arguments:   id, message identifier (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the ... service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetRollBackwards( 
	AsnInt *id )
{
	AsnInt type=UTB_SETROLLBACKWARDS_TYPE;
	return
		utRecv( id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
				 NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbSetCf
 *
 * Arguments:   id, message identifier (OUT)
 * 				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Get-Indication service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbSetCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex,
	AsnVbl *vars )
{
	AsnInt type=UTB_SETCF_TYPE;
	return
		utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL, 
				NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbInformRegRq
 *
 * Arguments:   ta, remote entity (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Inform registers-Request service
 *			 	primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbInformRegRq( 
	AsnOcts *ta )
{
	AsnInt type=UTB_INFORMREGRQ_TYPE;
	return
		utSend( NULL, &type, ta, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbInformUnRegRq
 *
 * Arguments:   ta, remote entity (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Inform unregisters-Request service
 *			 	primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbInformUnRegRq( 
	AsnOcts *ta )
{
	AsnInt type=UTB_INFORMUNREGRQ_TYPE;
	return
		utSend( NULL, &type, ta, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbInformRq
 *
 * Arguments:   id, message identifier (IN)
 *				ta, remote entity (IN)
 *				qos, quality of service (IN) 
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Inform-Request service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbInformRq( 
	AsnInt id,
	AsnOcts *ta, 	
	AsnInt qos, 
	AsnVbl *vars )
{
	/* AsnOcts *ta, toegevoegd aan de parameter list. (Rob) */
	AsnInt type=UTB_INFORMRQ_TYPE;
	return
		utSend( &id, &type, NULL, &qos, NULL, NULL, NULL, NULL, NULL, 
				&UT_CURRENTTIME, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbInformIn
 *
 * Arguments:   ta, remote entity (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Inform-Indication service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbInformIn( 
	AsnOcts *ta, 	
	AsnVbl *vars )
{
	AsnInt type=UTB_INFORMIN_TYPE;
	return
		utRecv( NULL, &type, ta, NULL, NULL, NULL, NULL, NULL, NULL,
				NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbInformCf
 *
 * Arguments:   id, message identifier (OUT)
 * 				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Inform-confirmation service 
 *				primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbInformCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex, 
	AsnVbl *vars )
{
	AsnInt type=UTB_INFORMCF_TYPE;
	return
		utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL,
				NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbTrapRegRq
 *
 * Arguments:   ta, remote entity (IN)
 *				qos, quality of service (IN) 
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Trap register-Request service
 *				primitive to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbTrapRegRq( 
	AsnOcts *ta )
{
	AsnInt type=UTB_TRAPREGRQ_TYPE;
	return
		utSend( NULL, &type, ta, NULL, NULL, NULL, NULL, NULL, 
				NULL, NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbTrapRq
 *
 * Arguments:   qos, qualityofservice (IN)
 *				vars, varbindinglist (IN)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Trap-Request service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbTrapRq( 
	AsnInt qos, AsnVbl *vars )
{
	AsnInt type=UTB_TRAPRQ_TYPE;
	return
		utSend( NULL, &type, NULL, &qos, NULL, NULL, NULL, NULL, NULL, 
			    NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utbTrapIn
 *
 * Arguments:   ta, remote entity (OUT)
 * 				cle, contextlocalentity (OUT)
 *				clt, contextlocaltime (OUT)
 *				vars, varbindinglist (OUT)
 *
 * Returns:		OK on success, and otherwise ERROR;
 *
 * Purpose:     This function provides the Trap-Indication service primitive 
 *				to the UT-APPL. 
 *
 * Remarks:		None
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utbTrapIn( 
	AsnOcts *ta, 
	AsnOcts *cle, AsnOid *clt, AsnVbl *vars )
{
	AsnInt type=UTB_TRAPIN_TYPE;
	return
		utRecv( NULL, &type, ta, NULL, NULL, NULL, NULL, NULL, cle, 
				clt, vars );
}



/* -----------------------------------------------------------------------
 */
