/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    ipm.c	(intermediate Protocol functions Module)
 *
 * Date:        September 1994
 *
 * Purpose:     This file realizes the ipm interface. The interface pro-
 *				vides the ipm service to the upm layer. The interface can
 *				be separated in two groups of functions.
 *
 *				General functions:
 *					upmOpen, upmInvoke and upmClose.
 *
 *				Service Primitive functions:
 *					ipm<X>Rq, upm<X>In, ipm<X>Rs and upm<X>Cf.
 *
 *				There relation is depicted below.
 *
 *													.....	
 *				 Open   Invoke   <X>Cf	<X>Rq 	<X>In	<X>Rs	Close
 *					^	^		 	^	|			^	|		^      UPM
 *				----|---|-----------|---|-----------|---|-------|---------
 *					|	|		 	|	v		 	|	v		|      IPM
 *									.   .............   .
 *									.....................
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */
#include <ipm.h> 

#define ipmMain main



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmGetRq	
 *
 * Arguments:	id, request identifier (IN)
 *				ta, transport address (IN)
 *				qos, quality of service (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm getrequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmGetRq(
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRq( &id, data->type = PDUS_GET_REQUEST, ta, qos, 
				  vi, le, lt, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	return lpmDataRq( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmGetRs	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm getrequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmGetRs(
	AsnInt id, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRs( &id, &src, &dst, &ctx ) != OK ) 
	{
		Error( "msg(%d) resolving <sub_agent> failed", id );
		return( ERROR ) ;
	}
	data->type = PDUS_RESPONSE ;
	return lpmDataRs( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmGetnextRq	
 *
 * Arguments:	id, request identifier (IN)
 *				ta, transport address (IN)
 *				qos, quality of service (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm getnextrequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmGetnextRq(
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRq( &id, data->type = PDUS_GET_NEXT_REQUEST, ta, qos, 
				  vi, le, lt, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	return lpmDataRq( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmGetnextRs	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm getnextresponse service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmGetnextRs(
	AsnInt id, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRs( &id, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed", id );
		return( ERROR ) ;
	}
	data->type = PDUS_RESPONSE ;
	return lpmDataRs( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmGetbulkRq	
 *
 * Arguments:	id, request identifier (IN)
 *				ta, transport address (IN)
 *				qos, quality of service (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm getbulkrequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmGetbulkRq(
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRq( &id, data->type = PDUS_GET_BULK_REQUEST, ta, qos, 
				  vi, le, lt, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	return lpmDataRq( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmGetbulkRs	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm getbulkresponse service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmGetbulkRs(
	AsnInt id, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRs( &id, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	data->type = PDUS_RESPONSE ;
	return lpmDataRs( id, dst, src, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmSetRq	
 *
 * Arguments:	id, request identifier (IN)
 *				ta, transport address (IN)
 *				qos, quality of service (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm setrequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmSetRq(
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRq( &id, data->type = PDUS_SET_REQUEST, ta, qos, 
				  vi, le, lt, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	return lpmDataRq( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmSetRs	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm setresponse service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmSetRs( 
	AsnInt id, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRs( &id, &src, &dst, &ctx ) != OK ) 
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	data->type = PDUS_RESPONSE ;
	return lpmDataRs( id, dst, src, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmInformRq	
 *
 * Arguments:	id, request identifier (IN)
 *				ta, transport address (IN)
 *				qos, quality of service (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm informrequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmInformRq(
	AsnInt id, AsnOcts *ta, AsnInt qos, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRq( &id, data->type = PDUS_INFORM_REQUEST, ta, qos, 
				  vi, le, lt, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	return lpmDataRq( id, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmInformRs	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm informresponse service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmInformRs(
	AsnInt id, 
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt src, dst, ctx ;

	if( yp2OutRs( &id, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	data->type = PDUS_RESPONSE ;
	return lpmDataRs( id, dst, src, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmTrapRq	
 *
 * Arguments:	id, request identifier (IN)
 *				ta, transport address (IN)
 *				qos, quality of service (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function realizes the ipm traprequest service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmTrapRq(
	AsnInt vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	AsnInt id = 0, src, dst, ctx ;

	if( yp2OutRq( &id, data->type = PDUS_SNMPV2_TRAP, NULL, NULL, 
				  vi, le, lt, &src, &dst, &ctx ) != OK )
	{
		Error( "msg(%d) resolving <src,dst,ctx> failed.", id );
		return( ERROR ) ;
	}
	return lpmDataRq( NULL, src, dst, ctx, data ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmOpen
 *
 * Arguments:	none
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function initializes the entire entity.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmOpen( )
{
	if( lpmOpen( ) != OK ) {
        Error( "unable to open LPM") ;
        return ERROR ;
	}
	if( ypOpen( ) != OK ) {
        Error( "unable to open YPM") ;
        return ERROR ;
	}
	if( upmOpen( ) != OK ) {
        Error( "unable to open UPM") ;
        return ERROR ;
	}
	return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmClose
 *
 * Arguments:	none
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function closes the entire entity.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmClose( )
{
	if( lpmClose( ) != OK ) {
        Error( "unable to close LPM") ;
        return ERROR ;
	}
	if( upmClose( ) != OK ) {
        Error( "unable to close UPM") ;
        return ERROR ;
	}
	return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	ipmMain
 *
 * Arguments:	none
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		mainloop
 *
 *				Algoritm:
 *
 *				1. Open the SPM, layer by layer; 
 *					- LPM (Lower Protocol Module), 
 *					- IPM (Intermediate Protocol Module) and 
 *					- UPM (Upper Protocol Module).
 *				2. Continuously process messages; top-down - and 
 *				   	bottum-up oriented.
 *				3. Close SPM.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
ipmMain( )
{
	AsnInt type, id, vi, src, dst, ctx, MARSHALL_T_ROSE = 1 ;
	AsnOcts *le = (AsnOcts *) pNewOct( 32 ), *ta = (AsnOcts *) pNewOct( 6 ) ;
	AsnOid *lt = pNewOid( 32 ) ;
	AsnData data ;
	
	if( ipmOpen( ) != OK ) return ERROR ;
	while( MARSHALL_T_ROSE )
	{
		/* TraceUser( ) ; */
		if( lpmLook( &type ) == OK )
		{
			if( type ==	PDUS_RESPONSE ) 
			{
				if( lpmDataCf( &id, &src, &dst, &ctx, &data ) != OK ) break ;

				MibGetContextLocalEntity( ctx, le ) ;
				MibGetContextLocalTime( ctx, lt ) ;
				MibGetContextViewIndex( ctx, &vi ) ;

				if( yp2InRs( &id, &type ) != OK ) break ;
				switch( type ) 
				{
					case PDUS_GET_REQUEST :
						upmGetCf( &id, &vi, le, lt, &data ) ; break ;
					case PDUS_GET_NEXT_REQUEST :
						upmGetnextCf( &id, &vi, le, lt, &data ) ; break ;
					case PDUS_GET_BULK_REQUEST :
						upmGetbulkCf( &id, &vi, le, lt, &data ) ; break ;
					case PDUS_SET_REQUEST :
						upmSetCf( &id, &vi, le, lt, &data ) ; break ;
        			case PDUS_INFORM_REQUEST :
						upmInformCf( &id, &vi, le, lt, &data ) ; break ;
				}
			} else {

				if( lpmDataIn( &id, &src, &dst, &ctx, &data ) != OK ) break ;

				MibGetContextLocalEntity( ctx, le ) ;
				MibGetContextLocalTime( ctx, lt ) ;
				MibGetContextViewIndex( ctx, &vi ) ;

				if( yp2InRq( &id, src, dst, ctx ) != OK ) break ;
				switch( type ) 
				{
					case PDUS_GET_REQUEST :
						upmGetIn( &id, &vi, le, lt, &data ) ;
						break ;

					case PDUS_GET_NEXT_REQUEST :
						upmGetnextIn( &id, &vi, le, lt, &data ) ; 
						 break ;

					case PDUS_GET_BULK_REQUEST :
						upmGetbulkIn( &id, &vi, le, lt, &data ) ; 
						break ;

					case PDUS_SET_REQUEST :
						upmSetIn( &id, &vi, le, lt, &data ) ; 
						break ;

        			case PDUS_INFORM_REQUEST :
						if( MibGetPartyTAddress( src, ta ) != OK )  break ;
						upmInformIn( &id, ta, &vi, le, lt, &data ) ;
					  	break ;

					case PDUS_SNMPV2_TRAP :
						upmTrapIn( &id, &vi, le, lt, &data ) ; 
						break ;
				}
			}
		}
		upmInvoke( ) ;
	}

	if( ipmClose( ) != OK ) return ERROR ;
	return OK ;
}



/* -------------------------------------------------------------------
 */
