/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    spmBER.h
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to Basic Encoding Rules module.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */
	    
#include <spmBer1.h>

static int SNMP_DEBUG_BER1 = 71;

/* 
 * -------------------------------------------------------------
 * Function:	BerDecodeV1Message
 * Arguments:	messageV1, ... (IN)
 *		version, .. (OUT)
 *		community, .. (OUT)
 *		tag, ... (OUT)
 *		pdu, ... (OUT)
 * Purpose:	Decodes the version and the community field of a
 *		SNMPv1 message. The pdu is returned in an octet string
 *		in BER format.
 * Remarks:	messageV1 starts with the tag 0x30
 * ----------------------------------------------------------------
 */
	STATUS
BerDecodeV1Message ( AsnOcts *messageV1, AsnInt *version, AsnOcts **community,
		   PDUsChoiceId *tag, AsnOcts **pdu)
{	
	char *buf = (char *) (messageV1->octs);
	MessageV1	result;
	AsnLen		bytesDecoded;
	jmp_buf		env;

	
	if( setjmp(env) == 0 ) 
		(void) BDecMessageV1 ( &buf, &result, &bytesDecoded, env );
	else return( ERROR );
	
	if( TRACE( SNMP_DEBUG_BER1, "BerDecodeV1Msg" ) == OK) {
		PrintMessageV1( stderr, &result, 3);
		fprintf( stderr, "\n" );
	}

	*version = result.version;
	(*community) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*community)->octs = result.community.octs;
	(*community)->octetLen = result.community.octetLen;

	*tag = (result.data)->choiceId;
	*pdu =  result.data->a.get_request;
		
	return OK ;
}


/* 
 * --------------------------------------------------------
 * Function:	BerDecodeV1Pdu
 * Arguments:	pdu, ... (IN)
 *		tag, ... (OUT)
 *		requestid, ... (OUT)
 *		errorstatus, ... (OUT)
 *		errorindex, ... (OUT)
 *		varbindlist, ... (OUT)
 * 		enterprise, ... (OUT)
 *		agent_addr, ... (OUT)
 *		generic_trap, ... (OUT)
 *		specific_trap, ... (OUT)
 *		time_stamp, ... (OUT)
 * Purpose:	This function decodes a SNMPv1 pdu with the type 'tag'
 *		requestid, errorstatus, errorindex and varbindlist are
 *		returned. In case of a trap enterprise, agent_addr,
 *		generic_trap, specific_trap and time_stamp are also
 *		returned.
 * Remarks:	none
 * --------------------------------------------------
 */
	
	STATUS
BerDecodeV1Pdu( AsnOcts *pdu, PDUsChoiceId tag, AsnInt *requestid,
		AsnInt *errorstatus, AsnInt *errorindex, VarBindList **varbindlist,
		AsnOcts **enterprise, AsnOcts **agent_addr, AsnInt *generic_trap,
		AsnInt *specific_trap, UAsnInt *time_stamp)
		 
{
	char *buf = (char *) (pdu->octs);
	PDUsV1		result;
	AsnTag		tagId;
	AsnLen		bytesDecoded;
	AsnLen elmtLen = (pdu->octetLen);
	jmp_buf		env;

	tagId = MAKE_TAG_ID ( CNTX, CONS, (tag));
	
	(void) BDecPDUsV1Content( &buf, tagId, elmtLen, &result, &bytesDecoded, env );
	if( TRACE( SNMP_DEBUG_BER1, "BerDecodeV1Msg" ) == OK ) {
		PrintPDUsV1( stderr, &result, 3);
		fprintf( stderr, "\n" );
	}
	
	if (tag == PDUS_TRAP)	{
	  *enterprise = &result.a.trap->enterprise;
	  *agent_addr = &result.a.trap->agent_addr;
	  *generic_trap = result.a.trap->generic_trap;
	  *specific_trap = result.a.trap->specific_trap;
	  *time_stamp = result.a.trap->time_stamp;
	  *varbindlist = result.a.trap->variable_bindings;
	} else { 
	  *requestid = result.a.get_request->request_id;
	  *errorstatus = result.a.get_request->error_status;
	  *errorindex = result.a.get_request->error_index;
	  *varbindlist = result.a.get_request->variable_bindings;
	}

	return OK ;
}


/* 
 * ---------------------------------------------------------------
 * Function:	BerEncodeV1Message
 * Arguments:	version, ... (IN)
 *		community, ... (IN)
 *		tag, ... (IN)
 *		requestid, ... (IN)
 *		errorstatus, ... (IN)
 *		errorindex, ... (IN)
 *		variablebindings, ... (IN)
 *		messageV1, ... (OUT)
 * Purpose:	Construct a SNMPv1 message
 * Remarks:	none
 * --------------------------------------------------------------
 */

	STATUS
BerEncodeV1Message (AsnOcts **messageV1, AsnInt version, AsnOcts *community,
		  PDUsChoiceId tag, AsnInt requestid, AsnInt errorstatus,
		  AsnInt errorindex, VarBindList *variablebindings,
		  AsnOcts *enterprise, AsnOcts *agent_addr, AsnInt generic_trap,
		  AsnInt specific_trap, UAsnInt time_stamp)
{
	char 		*encBuf;
	char		*encData;
	PDUsV1		*pdu;
	MessageV1	*message;
	AsnLen		pduLen,messageLen;
	int encSize = 64*1024;

	
	pdu = (PDUsV1 *) AllocPDUsV1( tag, enterprise->octetLen, agent_addr->octetLen);

	if (tag == PDUS_TRAP) {
	  pdu->a.trap->enterprise.octs = enterprise->octs;
	  pdu->a.trap->enterprise.octetLen = enterprise->octetLen;
	  pdu->a.trap->agent_addr.octs = agent_addr->octs;
	  pdu->a.trap->agent_addr.octetLen = agent_addr->octetLen;
	  pdu->a.trap->generic_trap= generic_trap;
	  pdu->a.trap->specific_trap = specific_trap;
	  pdu->a.trap->time_stamp = time_stamp;
	  pdu->a.trap->variable_bindings = variablebindings;
	} else {
	  pdu->a.get_request->request_id = requestid;
	  pdu->a.get_request->error_status = errorstatus;
	  pdu->a.get_request->error_index = errorindex;
	  pdu->a.get_request->variable_bindings = variablebindings;
	}
	
	encData = (char*) Asn1Alloc(encSize);
	encBuf = encData + encSize;
	pduLen = BEncPDUsV1( &encBuf, pdu);

	if( TRACE( SNMP_DEBUG_BER1, "BerEncodeV1Pdu" ) == OK ) {
	 	PrintPDUsV1( stderr, pdu, 3);
		fprintf( stderr, "\n" );
	}
	  
	message = (MessageV1 *) AllocMessage(tag, pduLen, community->octetLen);
	message->data->a.get_request->octs = encBuf;
	message->data->a.get_request->octetLen = pduLen;

	message->version = version;
	message->community.octs = community->octs;
	message->community.octetLen = community->octetLen;
	
	encBuf = message->data->a.get_request->octs;
	
	messageLen = BEncMessageV1( &encBuf, message);

	if( TRACE( SNMP_DEBUG_BER1, "BerEncodeV1Message" ) == OK ) {
		PrintMessageV1( stderr, message, 3);
		fprintf( stderr, "\n" );
	}

	*messageV1 = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*messageV1)->octs = encBuf;
	(*messageV1)->octetLen = messageLen;
	
	return OK ;
}

/* ----------------------- end of file spmBer1.c ------------------------ */

