/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    per_len.c
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to PER modules.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */
#include <snacc/asn_config.h>
#include <snacc/asn_incl.h>
#include <perv2/per_buf.h>
#include <perv2/per_len.h>
#include <snmpTypes.h>



/* ----------------------------------------------------
 * Function:	PerEncLenCons.
 * Arguments:	length, ...... (IN)
 * 		b, .......... (IN/OUT)
 * 		shift, ........ (IN)
 * Purpose:	Encodes length in ONE byte this is needed
 *		for the CHOICE types & the OCTET STRING
 *		SIZE(1|4..21) types. This function can 
 *		only be applied in the ALIGNED variant
 *		because I'll make a byte of the length
 *		every time :). For the UNALIGNED variant
 *		there must be several functions that
 *		return a bit-field (NO octet-alignment)
 * Remarks:	VarBind, SimpleSyntax, ApplicationSyntax,
 *		and PDU make use of this function.
 *		
 * -----------------------------------------------------
 */

	STATUS
PerEncLenCons(BUF_TYPE b, UAsnInt value, UAsnInt shift)
{
 unsigned long int	dataCpy;

 dataCpy = value;

 dataCpy <<= shift;
 if (dataCpy > 255)
 {
	fprintf(stderr,"The length is bigger than 2^8-1");
	return( ERROR );
 }
 else
 {
   PerPutByte(b, dataCpy);
 }
 return( OK );
}
/*-----------------------------------------------------------
 * Function:	PerEncLenAddr
 * Arguments:	b, (OUT/IN)
 *		choiceVal (IN)
 *		length (IN)
 * Purpose:	Per encodes the choice and the length
 *		in 2 bytes. This function os only applied
 *		by the NsapAddress application-wide stuff
 *		Because the length of this octet string
 *		is limited (1..21)
 * Remarks:	e.g. If the actual length is 12 than the
 *		encoded length field is 11.
 * -----------------------------------------------------------
 */
	STATUS
PerEncLenAddr(BUF_TYPE b, UAsnInt choiceVal, UAsnInt length)
{
 unsigned long int	dataCpy;

 dataCpy = choiceVal;

 dataCpy <<= 5;/* creating 5 bits space for length */
 dataCpy |= (length-1);
 dataCpy <<= 4; /* padding 4 bit of zero */
 if (dataCpy > 65536)
 {
	fprintf(stderr,"The length is bigger than 2^8-1");
	return( ERROR );
 }
 else
 {
   PerPutByte(b, dataCpy);
   dataCpy >>= 8;
   PerPutByte(b, dataCpy);
   }
 return( OK );
}

/* ------------------------------------------------------------------
 * Function:	PerDecLenCons
 * Arguments:	value, ........ (OUT)
 *		b , ........... (IN)
 *		shift, ...... (IN)
 * Purpose:	Decode ONE byte and give back the value of 
 *		it into length.
 * Remarks:	This functions can only be used for constrainted
 *		length fields (<256, see functions above!!!!!)
 * 		The value will be shifted a number of bytes
 * ---------------------------------------------------------------------
 */
	STATUS
PerDecLenCons(BUF_TYPE b, UAsnInt *value, UAsnInt shift)
{
 unsigned long int	byte = 0;

 byte = (unsigned long int) PerGetByte( b );
 *value= (byte >> shift);
 return( OK );
}

/* ------------------------------------------------------------------
 * Function:	PerPeekLenCons
 * Arguments:	value, ........ (OUT)
 *		b , ........... (IN)
 *		shift, ...... (IN)
 * Purpose:     Does the same as PerDecLenCons, it doesn't shift
 *		the "b" pointer.
 * Remarks:	This functions can only be used for constrainted
 *		length fields (<256, see functions above!!!!!)
 * 		The value will be shifted a number of bytes
 * ---------------------------------------------------------------------
 */
	
	
	STATUS
PerPeekLenCons(BUF_TYPE b, UAsnInt *value, UAsnInt shift)
{
 unsigned long int	byte = 0;

 byte = (unsigned long int) PerPeekByte( b );
 *value= (byte >> shift);
 return( OK );
}
/*-----------------------------------------------------------
 * Function:	PerDecLenAddr
 * Arguments:	b, (IN)
 *		choiceVal (OUT)
 *		length (OUT)
 * Purpose:	Per decodes the choice and the length
 *		out of  2 bytes. This function os only applied
 *		by the NsapAddress application-wide stuff
 *		Because the length of this octet string
 *		is limited (1..21)
 * Remarks:	Nope
 * -----------------------------------------------------------
 */
	
	STATUS
PerDecLenAddr(BUF_TYPE b, UAsnInt *choiceVal, UAsnInt *length)
{
 unsigned long int	byte = 0;

 byte = (unsigned long int) PerGetByte( b );
 byte <<= 8;
 byte |= (unsigned long int) PerGetByte( b );
 
 byte >>= 4; /* remove the padding bits */
 *length = ( byte & (0x1F) );/* take the last 5 bits of byte */
 *length++; /* the actual length is 1 higher */
 byte >>= 5;
 *choiceVal = byte;
 return( OK );
}

/* --------------------------------------------------
 * Function:	PerDecLenDeter.
 * Arguments:	length, UAsnInt (OUT)
 *		b, ....,  (IN)
 *		choice, AsnInt (IN)
 * Returns:	Length of the decoded field in octets	
 * Purpose:	Constuct a length from the length determinant
 *		field. What follows will be an 
 *		octet-aligned bit field
 *		(for SEQUENCE OF, OCTET STRING, BIT STRING)
 *		(see 9.8 PER-CD). 
 * Remarks:	The choice is between SEQUENCE OF,
 *		OCTET STRING, BIT STRING (bitLen <128))
 * ----------------------------------------------------
 */
	AsnLen	
PerDecLenDeter(BUF_TYPE b, UAsnInt *length, AsnInt choice)
{
 unsigned long int 	retVal = 0;
 unsigned long int	byte = 0;
 unsigned long int	largeVal = 0;
 AsnLen			itemLen = 0;

#define ONE_MASK 	0x80  /* bit 8 is set to one */
#define TWO_MASK	0xc0  /* bit 8 & 7 are set to one */

 byte = (unsigned long int) PerGetByte( b );

 if ((byte & ONE_MASK) != ONE_MASK) /* lenght is smaller than 128 */
 {
	*length = byte;
	return(1);
 }
 if ((byte & TWO_MASK) != TWO_MASK) /* lenght is smaller than 16K */
 {
	/* 
	 * bits 7 to 1 will from original byte will
	 * be copied into retVal
	 *
	 */ 
	retVal = (byte & (~ONE_MASK));
	retVal <<= 8; /* shift retVal 1 byte to left */
	byte = PerGetByte(b);
	retVal |= byte; /* copy next byte into retVal */
	*length = retVal; 
	return(2);
 }
 else   /* length is equal or bigger than 16K */
 {
	/* 
	 * bits 6 to 1 will from original byte will
	 * be copied into retVal
	 *
	 */
	retVal = (byte & (~TWO_MASK));
	
	largeVal = retVal * 16384; /* multiply the retVal with 16K see 9.8.12
				    * PER Committee Draft
				    */
	*length = largeVal;
	
	/* the length is bigger than 16K so first write the length 
	   determinant with the fragmentation appended by the remaining
	   part of it !!!!!!!!!!!!!!!!!!! */  
	return(3); /* length of the field was 1!!!*/
 }
 return( OK );
}

/* --------------------------------------------------
 * Function:	PerEncLenDeter.
 * Arguments:	length, UAsnInt (IN)
 *		b, ....,  (IN/OUT)
 *		choice, AsnInt (IN)
 * Returns:	Length of the encoded field.
 * Purpose:	Constuct a length determinant field
 *		to be prepended to a field-list. 
 *		(see 9.8 PER-CD)
 * Remarks:	The choice is between SEQUENCE OF,
 *		OCTET STRING, BIT STRING (bitLen < 128)
 *		if lenght is bigger than 16K, the return
 *		value will be 3 and fragmentation will be
 *		there the length value contains the remaining
 *		length of the content!!!!
 * ----------------------------------------------------
 */
	AsnLen	
PerEncLenDeter(BUF_TYPE b, UAsnInt *length, AsnInt choice)
{
  unsigned long int	mcount;
  unsigned long int    	dataCpy;
  AsnLen		itemLen;

  itemLen = 0;
  dataCpy = *length;

  if (dataCpy < 128)  
  {
    PerPutByte(b,dataCpy);/* set LSB byte into buf */
    /* if length is smaller than 128 then MSbit is always set!*/
    return(1);
   }
  else
  {
    if (dataCpy < 16384)
    {
      PerPutByte(b,dataCpy);
      dataCpy >>= 8; /* shift dataCpy 1 byte to the right */
      dataCpy |= ONE_MASK;
      /*
       * bit 8 of the LSByte of dataCpy is set to one 
       * bit 7 of the LSByte of dataCpy is set to zero
       *
       */
      PerPutByte(b,dataCpy);
      return(2);
     }
    else 
    {
      if (dataCpy >= 16384) /* more than 16 K */ 
      {
	mcount = dataCpy/16384;
	if (mcount > 4)
	{
	  mcount = 4;
	}
	dataCpy -= (mcount*16384);

	mcount |= TWO_MASK;
        /*
         * bit 8 of the LSByte of mcount is set to one 
         * bit 7 of the LSByte of mcount is set to one 
         *
         */

	PerPutByte(b,mcount); 
	*length = dataCpy;
	return( 3 );	
       } /* end if */
     } /* end else if */
  } /* end else if */
  return( OK );
}/* PerEncLenDeter */

