/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    lpm.c	(Lower Protocol functions Module)
 *
 * Date:        September 1994
 *
 * Purpose:     This file realizes the lpm interface. The interface pro-
 *				vides the lpm service to the ipm layer. The interface can
 *				be separated in two groups of functions.
 *
 *				General functions:
 *					lpmOpen( ... ), lpmLook( ... ) and lpmClose( ... ).
 *
 *				Service Primitive functions:
 *					lpmDatRq, lpmDataIn, lpmDataRs and lpmDataCf.
 *
 *				There relation is depicted below.
 *
 *													 ....	
 *				 Open   Look 	DataCf	DataRq 	DataIn	DataRs	Close
 *					|	^			 ^	|			 ^	|		|      IPM
 *				----|---|------------|--|------------|--|-------|---------
 *					v	|			 |	v			 |	v		v      LPM
 *									 .  ..............  .
 *									 ....................
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */
#include <lpm.h> 

#define lpmWrap SpmWrapV2Message		                 /* blubber ... */
#define lpmUnwrap SpmUnwrapV2Msg				         /* blubber ... */

TMESSAGE tm;

AsnInt _id, _l, _r, _c ;
AsnData _d ;

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	lpmOpen
 *
 * Arguments:	none 
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function opens and initializes the lpm module.
 *				All relevant "lower-level" modules are opened.
 *				Secondly, the network ports as specified by the PartyMib
 *				Module will be opened.
 *
 * Remark:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
lpmOpen()
{
	AsnOid	*domain=pNewOid( 9 ) ;
	AsnOcts	*address=pNewOct( 6 ) ;
	AsnInt	src=0, dst=0, ctx=0, acl=0, local=0, d=0 ;

	InitNibbleMem( 32*1024, 512 );

    if( SpmInitPartyMIB() != OK ) {
        Error( "unable to open PMM (PartyMibModule)" ) ;
        return ERROR ;
    }
    if( SpmInitSnmpV2Mib() == ERROR ) {
        Error( "unable to open SMM (SnmpV2MibModule)" ) ;
        return ERROR ;
    }
    if( SpmInitTim( ) != OK ) {
        Error( "unable to open TIM (TransportInterfModule)" ) ;
        return ERROR ;
    }
    if( SpmInitBer( ) != OK ) {
        Error( "unable to open CM (CodingModule)" ) ;
        return ERROR ;
    }

	/*
	 *	Open transport ports ...
	 */
	if (MibResetAclEntry() == OK ) 
		while ( MibGetAclEntry( &src, &dst, &ctx, &acl ) == OK ) {

			if( ( MibGetPartyLocal( src, (TruthValue *) &local ) != OK ) || 
	    	    ( local != true ) ) continue ;
			if( MibGetPartyTDomain( src, domain ) != OK ) continue ;
			if( DomainConvert( domain, &d ) != OK ) continue ; 
			if( MibGetPartyTAddress( src, address ) != OK ) continue ;
			if( TransportOpen( d, address->octs ) != OK ) continue ;
		}
	pFreeOid(domain) ; pFreeOct(address) ;	
	return OK;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	lpmClose
 *
 * Arguments:	none 
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function closes this module (in an gracefull way).
 *
 * Remark:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
lpmClose( )
{
	SpmCloseBer( ) ;
    SaveSnmpStats( ) ;
    /* SavePartyMIB( ) ; */
    /* CloseTransportConnections( ) ; might be obsolete! */
	ShutdownNibbleMem( ) ;
    return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	lpmLook
 *
 * Arguments:	type, indicates request or response (OUT)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function monitors the network layer and receives
 *				a network message, and administrates this message
 *				by the triple <id,tdomain,taddress>.
 *
 * Remark:		la, local address.
 *				ra, remote address.
 *				td, transport domain
 *				tm, transport message.
 *				d, dummy
 *
 * -------------------------------------------------------------------
 */
	STATUS
lpmLook( AsnInt *type )
{
	AsnOcts m ;
	static AsnOcts *la, *ra ;
	AsnInt td ;

	/*	Commented, because vb must be permanent available in ind(ex)
	 *  reg(ister)list for specific NoSuchObject situations. EvH.
 	 *
	ShutdownNibbleMem( ) ;
	InitNibbleMem( 32*1024, 512 );
	 */

	la = pNewOct( 6 );  ra = pNewOct( 6 );
	if( TransportReceive( (TDOMAIN*)&td, la->octs, ra->octs, 
						  &tm ) != OK ) 
	{
		pFreeOct( la ) ;
		pFreeOct( ra ) ;
		return ERROR ;
	}
	m.octetLen = tm.length; m.octs = tm.buf;
	if( lpmUnwrap( m, &_l, &_r, &_c, &(_d.type), &_id, &(_d.etype), 
				   &(_d.eindex), &(_d.vbl) ) != OK )
	{
		pFreeOct( la ) ;
		pFreeOct( ra ) ;
		return ERROR ;
	}
	*type = _d.type ;
	if( *type != PDUS_RESPONSE )
	{
	 	yp3InRq( &_id, td, la, ra );
	} else {
		pFreeOct( la ) ;
		pFreeOct( ra ) ;
	}
	return( OK ) ;
}


 
/* 
 * -------------------------------------------------------------------
 *
 * Function:	lpmDataRq	
 *
 * Arguments:	id, request identifier (IN)
 *				l, r, local and remote parties (IN)
 *				c, context (IN)
 *				d, data (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		The lpm user submits an request message, identified 
 *				by id, to an remote lpm user, addressed by <l,r,c>, and
 *				contains data d.	
 *
 *				Algoritm:
 *				- Map srcparty upon <domain,local transport address>
 *				- Map dstparty upon <domain,remote transport address>
 *				- Wrap pdu-elements in communication message m.
 *				- Send communication message m
 *
 * Remark:		la, local address.
 *				ra, remote address.
 *				td, transport domain
 *
 * -------------------------------------------------------------------
 */
	STATUS
lpmDataRq( AsnInt id, AsnInt l, AsnInt r, AsnInt c, AsnData d )
{
	AsnOcts *la = NewOct( 6 ), *ra = NewOct( 6 ), *td = NewOct( 16 ) ;
	AsnOcts *m ;
	
	if( yp3OutRq( l, td, la ) != OK ) return ERROR ;
	if( yp3OutRq( r, td, ra ) != OK ) return ERROR ;
	if( lpmWrap( l, r, c, d.type, id, d.etype, d.eindex, d.vbl, &m ) != OK )
		return ERROR ;
	if( TransportSend( UDPDOMAIN, la->octs, ra->octs, m ) != OK ) return ERROR ;
	return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	lpmDataIn	
 *
 * Arguments:	sid, reply identifier (IN)
 *
 *				l, r, local and remote parties (IN)
 *				c, context (IN)
 *
 *				d, data (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function unwraps an received message and supplies
 *				pdu elements to the lpm user as an indication.
 *
 * Remark:		m, transport message.
 *
 * -------------------------------------------------------------------
 */
	STATUS
lpmDataIn( AsnInt *id, AsnInt *l, AsnInt *r, AsnInt *c, AsnData *d )
{
	*id = _id ;
	*l = _l ; *r = _r ; *c = _c ;
	*d = _d ;

	return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	lpmDataRs	
 *
 * Arguments:	id, reply identifier (IN)
 *
 *				l, r, local and remote parties (IN)
 *				c, context (IN)
 *
 *				d, data (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		The lpm user submits an response message, identified 
 *				by id, to an remote lpm user, addressed by <l,r,c>, and
 *				contains data d.	
 *
 *				Algoritm:
 *				- Map srcparty upon <domain,local address>
 *				- Map reply identifier upon <domain,remote address>
 *				- Wrap pdu-elements in communication message m.
 *				- Send communication message m
 *
 * Remark:		la, local address.
 *				ra, remote address.
 *				td, transport domain
 *
 * -------------------------------------------------------------------
 */
	STATUS
lpmDataRs( AsnInt id, AsnInt l, AsnInt r, AsnInt c, AsnData d )
{
	AsnOcts *la=NULL, *ra=NULL, *td=NewOct(32) ;
	AsnOcts *m;
	
	if( yp3OutRs( &id, &td, &la, &ra ) != OK ) return ERROR ;
	if( (la == NULL) || (ra == NULL) ) return ERROR ;
	if( lpmWrap( l, r, c, d.type, id, d.etype, d.eindex, d.vbl, &m ) != OK )
		return ERROR ;
	if( TransportSend( UDPDOMAIN, la->octs, ra->octs, m ) != OK ) return ERROR ;
	pFreeOct( la ) ; pFreeOct( ra ) ;
	return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	lpmDataCf	
 *
 * Arguments:	sid, reply identifier (IN)
 *
 *				l, r, local and remote parties (IN)
 *				c, context (IN)
 *
 *				d, data (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function unwraps an received message and supplies
 *				pdu elements to the lpm user as an confirmation.
 *
 * Remark:		m, transport message.
 *
 * -------------------------------------------------------------------
 */
	STATUS
lpmDataCf( AsnInt *id, AsnInt *l, AsnInt *r, AsnInt *c, AsnData *d )
{
	*id = _id ;
	*l = _l ; *r = _r ; *c = _c ;
	*d = _d ;
	return OK ;
}




/* -------------------------------------------------------------------
 */
