/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename: 	spmUDP.c
 *
 * Date:	May 1994
 *
 * Purpose:	implementation of transport level interface with sockets
 *
 * Author(s):	Vincent Berkhout, Frederic P. H. van Hengstum 
 *
 * -----------------------------------------------------------------------
 */

#include <spmSocket.h>

/*
 * 	local variables
 */
static BOOLEAN SNMP_DEBUG_SOCKET = 31;

/* 
 * 	First the common calls will be handled which are used by both TCP and UDP.
 */


/*------------------------------------------------------------------------*/
	STATUS
SocketCheckInputs( allHandles, activeHandles ) 
int 	*allHandles,	/* in, all available handle/filedescriptors */
	*activeHandles;	/* in, all handles/filedescriptors that are active */ 
/* 
 * Purpose:
 *	This call checks whether data is ready to be received on the 
 *	given handles "allHandles". The handles for which data is 
 *	returned by "activeHandles".
 * Remarks:
 * 	STATUS returns nrActiveHandles on success!
 */
{	
	int	i, j, nrActiveHandles, maxFiles;	
	fd_set	readset;
	struct 	timeval	timeout;
	struct	rlimit	r ;	/* maximimum number of fildescriptors available */
	static BOOLEAN init = FALSE;

	if ( ! init ) {
		init = TRUE;
	}
/*
 *	initialize the readset and the empty time value
 */
	FD_ZERO( &readset ) ;
	timeout.tv_sec = timeout.tv_usec = 0 ;
	(void) getrlimit( RLIMIT_NOFILE, &r );
	maxFiles = r.rlim_cur ;	
/* 
 * 	incorporate handles (integer) into readset ("filedescriptor set") 
 */
	for (i = 0; i < maxFiles ; i++) {
		if ( allHandles[i] > -1 ) 
			FD_SET( allHandles[i] , &readset ) ;
		else 	break;
	}	/* endfor i to maxFiles */
/* 
 * 	now perform the select
 */
	nrActiveHandles = select( maxFiles, (fd_set *)&readset, (fd_set *)NULL, (fd_set *)NULL, &timeout);

/* 
 * 	and fill the array of int (activeHandles) with the active handles
 *	(If a handle is active it is copied from allHandles to activeHandles)
 */
	if (nrActiveHandles <= 0)
		activeHandles[0] = -1 ;
	else
		for( i = 0, j = 0; i < maxFiles; i++ ) 
			if ( ( allHandles[i] > -1 ) && 
			   FD_ISSET( allHandles[i], &readset ) )
				activeHandles[j++] = allHandles[i] ;

	return nrActiveHandles ;

}	/* end of function  SocketCheckInputs */


/*------------------------------------------------------------------------*/
	STATUS
SocketClose ( socket ) 
int 	socket;			/* in, filedescriptor to be closed */
/*
 * Purpose:
 * 	The call simply closes the socket/filedescriptor with the systemcall close
 * Remarks:
 */
{
	if ( close ( socket )  < 0) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketClose'close") ;
		return ERROR ;
	}	/* endif close  */
	return OK ;
}	/* end of function SocketClose */


/*------------------------------------------------------------------------*/
	STATUS
SocketPacketOpen( localTAddress, dataSocket) 
u_char 	*localTAddress;	/* in, the port on which to listen */
int	*dataSocket;	/* out, a socket on which data can be received */
/*
 * Purpose:
 * 	This call performs an open for udp at the local side . It derives the
 * 	portnumber from the localTAddress parameter. It subsequently creates
 *	a socket, binds it to the portnumber and goes into "listen" mode.
 * Remarks:
 */
{
	int	msgSize = PACKETSIZE ;
	long	hostNr ;
	short	portNr ;
	struct 	sockaddr_in	newSock ;
/* 
 * 	first  open a socket with some initial values
 */
	*dataSocket = socket( PF_INET,  SOCK_DGRAM, 0) ;
	if( *dataSocket < 0)   {
		if (SNMP_DEBUG_SOCKET)
		perror( "spmSocket'SocketPacketOpen'socket") ;
		return ERROR ;
	}	/* endif socket creation */
/*
 *	set the packetsize as big as possible, preferably at 64 K
 */ 
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_SNDBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketPacketOpen'setsockoptSNDBUF") ;
	} 	/* endif set SNDBUF */
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_RCVBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketPacketOpen'setsockoptRCVBUF") ;
	} 	/* endif set RCVBUF */

/*
 *	get the port number from transport address
 */
  	(void) memcpy( &(hostNr), (const void *) localTAddress,  4 );
	(void) memcpy( &(portNr), (const void *) (localTAddress + 4) , 2);
/*
 *	fill in the socket properties
 */
	newSock.sin_family = AF_INET ; 			/* it is an internet address */
	newSock.sin_addr.s_addr = INADDR_ANY ;		/* address wild card */
	newSock.sin_port = htons( portNr ) ; 		/* assign the port number*/
/* 
 * 	now bind the socket to the transport address which was just filled in
 */
	if (bind( *dataSocket , (struct sockaddr *)&newSock, sizeof( newSock ) ) < 0 ) {
		if (SNMP_DEBUG_SOCKET)
		perror( "spmSocket'SocketPacketOpen'bind" ) ;
		(void) close (*dataSocket ) ;
		return ERROR  ;
	}	/* endif bind */
	return OK ;
} 	/* end of function SocketPacketOpen */


/*------------------------------------------------------------------------*/
	STATUS	
SocketPacketSend( dataSocket, remoteTAddress, message)
int	*dataSocket;		/* in, the datasocket to send the data to */
u_char	*remoteTAddress;	/* in, the remote address to be incorparated in the message */
char	*message;		/* in, the data to be send*/
/* 
 * Purpose:
 * 	This function sends the data to the remote address. 
 * 	After conversion of the address, the messagesize is determined and the
 *	message is send. 
 * Remarks: 
 */
{
	int  	nrWritten,
		addressSize,
		messageSize;
	long	hostNr;
	short	portNr;
	struct	sockaddr_in 	remote;
/* 
 * 	first convert the transport address 
 */
	(void) memset( (char *)&remote, '\0', sizeof(struct sockaddr_in)) ;
  	(void) memcpy(&hostNr ,(const void *)remoteTAddress, 4 );
	(void) memcpy(&portNr, (const void *)(remoteTAddress + 4), 2);
/*
 * 	fill in the remote address
 */
	remote.sin_family = AF_INET;   		/* tcp belongs to the internet address family*/
	remote.sin_addr.s_addr = htonl(hostNr);	/* convert hostNr to network byte order */
	remote.sin_port = htons(portNr);	/* convert portNr to network byte order */
	(void) memcpy( &messageSize, message, 4 );
/* 
 * 	and finally send the data 
 */
	addressSize = sizeof (remote) ;
	nrWritten =  sendto( *dataSocket, message+4, messageSize, 0, 
			     (struct sockaddr *)&remote, addressSize ) ;

	TRACE( SNMP_DEBUG_SOCKET, "%d bytes written on socket %d", 
			nrWritten, *dataSocket) ;

	if ( nrWritten < 0 ) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketPacketSend'sendto") ;
		return FAILED ;
	}	/* end if nrWritten */
	return OK ;
} 	/* end of function SocketStreamSend */


/*------------------------------------------------------------------------*/
	STATUS	
SocketPacketReceive( dataSocket, remoteTAddress, message)
int	*dataSocket; 		/* in, socket to receive on  */
u_char	*remoteTAddress;  	/* out, remote address of the sending peer */
char	*message;		/* out, the data that has been send by the peer */
/*
 * Purpose:
 *	Reads the data from the given socket and returns the message AND peer address
 * Remarks:
 */
{	
	int  	nrReads,
		addressSize,
		msgSize = PACKETSIZE ;
	long	hostNr ;
	short	portNr ;
	struct	sockaddr_in	remote;

	addressSize = sizeof(struct sockaddr_in) ;

	nrReads = recvfrom( *dataSocket, message+4, msgSize, 0,
			    (struct sockaddr *)&remote, &addressSize ) ;

	TRACE( SNMP_DEBUG_SOCKET, "%d bytes read on socket %d", 
			nrReads, *dataSocket) ;

	if (nrReads < 0) {
		TRACE(SNMP_DEBUG_SOCKET, "recvfrom non existing socket") ;
		return ERROR ;
	}	/* endif nrReads */

	(void) memcpy( message, &nrReads, 4 );
	hostNr = ntohl(remote.sin_addr.s_addr) ;
	portNr = ntohs(remote.sin_port) ;
 	(void) memcpy( (void *) remoteTAddress, (const void *)&hostNr,   4 );
	(void) memcpy( (void *) (remoteTAddress + 4), (const void *)&portNr,  2);
	return OK ;
	
} 	/* end of function SocketPacketReceive */

/*---------------------- end of file spmSocket.c -----------------------------*/

