/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    yp3.c (yellow pages module)
 *
 * Date:        September 1994
 *
 * Purpose:     This file implements the local - and remoterequest-lists.
 *
 * Author:      Frederic P. H. van Hengstum, Vincent Berkhout.
 *
 * Remarks: 	This file implements the transport level addressing,
 *				accomplished by the tuple (taddress, tdomain) information.
 *
 *
 *							OUT	       Rq
 *						               ^
 *						----|-|--------|---- network layer
 *						    v v	
 *						   Rq Rs	   IN
 *
 * -----------------------------------------------------------------------
 */
#include <yp3.h>




/* 
 * -------------------------------------------------------------------
 *
 * Function:    yp3InRq
 *
 * Arguments:   id, request identifier and handle to stored info (IN/OUT)
 *				tdomain, transport domain (IN)
 *				la, local address (IN) 
 *				ra, remote address (IN) 
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function registers the transport domain and the
 *				transport address from an incoming message. The 
 *				incoming message is timestamp and an timeouttimer is 
 *				set.
 *
 * Remarks:		none.
 *
 * -------------------------------------------------------------------
 */
	STATUS
yp3InRq( AsnInt *id, AsnInt tdomain, AsnOcts *la, AsnOcts *ra )
{
	AsnInt i;
	static AsnInt init=0;
	struct timeval tp;

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR;
	if( !init ) {
		for( i = 0; i < YP3LISTMAX; i++ )
		{
			yp3list[i].tdomain = NULL ;
			yp3list[i].la = NULL ;
			yp3list[i].ra = NULL ;
			yp3list[i].timestamp = 0 ;
		}
		init = 1 ;
	}
	for( i=1; i<YP3LISTMAX; i++ )
		if( yp3list[i].id == 0 ) 
		{
			yp3list[i].id = *id ;
			yp3list[i].tdomain = tdomain ;
			yp3list[i].la = la ;
			yp3list[i].ra = ra ;
			yp3list[i].timestamp = tp.tv_sec ;
			*id = i ;
			return OK ;
		}
	return ERROR ;
}
		

/*  
 * -----------------------------------------------------------------------
 *
 * Function:	yp3OutRq
 *
 * Arguments:	dst, destination party (IN)
 *				tdomain, transport domain (OUT)
 *				ra, remote address (OUT)
 *
 * Returns:		OK on success, otherwise ERROR.
 *
 * Purpose:		This function maps an destination party upon an transport 
 *				domain and transport address.
 *
 *				Firstly, the local yp3 list is updated. Expired entries are
 *				simply removed. 
 *				Afterwards, the given partyentry indexes the desired 
 *				transport domain and transport address.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp3OutRq( AsnInt dst, AsnInt *tdomain, AsnOcts *ra )
{
	AsnInt i ;
	struct timeval tp ;

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR ;
	for( i = 0; i < YP3LISTMAX; i++ )
		if( yp3list[i].timestamp + YP3LISTTIMEOUT < tp.tv_sec )
		{
			yp3list[i].tdomain = NULL ;
			yp3list[i].la = NULL ;
			yp3list[i].ra = NULL ;
			yp3list[i].timestamp = 0 ;
		}

	if( MibGetPartyTDomain( dst, tdomain ) != OK ) return ERROR ;
	if( MibGetPartyTAddress( dst, ra ) != OK ) return ERROR ;

	return OK ;
}



/*  
 * -----------------------------------------------------------------------
 *
 * Function:	yp3OutRs
 *
 * Arguments:  	id, handle and request identifier (IN/OUT)
 *				tdomain, transport domain (OUT)
 *				la, local address (OUT) 
 *				ra, remote address (OUT) 
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:		This function maps an handle upon an transport domain and 
 *				transport address.
 *				Besides that, the yp3list is updated by removing the expired
 *				entries.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp3OutRs( AsnInt *id, AsnInt *tdomain, AsnOcts **la, AsnOcts **ra )
{
	AsnInt i ;
	struct timeval tp ;

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR ;
	for( i = 0; i < YP3LISTMAX; i++ )
		if( yp3list[i].timestamp + YP3LISTTIMEOUT < tp.tv_sec ) 
		{
			yp3list[i].tdomain = NULL ;
			if( yp3list[i].la != NULL ) pFreeOct( yp3list[i].la ) ;
			yp3list[i].la = NULL ;
			if( yp3list[i].ra != NULL ) pFreeOct( yp3list[i].ra ) ;
			yp3list[i].ra = NULL ;
			yp3list[i].timestamp = 0 ;
		}

	if( (*id<0) || (*id>YP3LISTMAX) || (yp3list[*id].id==0) ) return ERROR ;

	i = *id ;
	*id = yp3list[i].id ;
	*tdomain = yp3list[i].tdomain ;
	*la = yp3list[i].la ;
	*ra = yp3list[i].ra ;

	yp3list[i].tdomain = NULL ;
	yp3list[i].la = NULL ;
	yp3list[i].ra = NULL ;
	yp3list[i].timestamp = 0 ;

	return OK ;
}



/* -----------------------------------------------------------------------
 */
