 /* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    indreg.c
 *
 * Date:        September 1994
 *
 * Purpose:     This file implements the index regstration, meaning, upon
 *				receipt of an pdu from the ipm layer the corresponding
 *				variablebindinglist is segmented in individual varbindings.
 *
 *				Every variable(binding) is checked against its view.
 *				Afterwards, the variable is checked against the
 *				mibreg list to figure out, by comparing the triples
 *				<var,cle,clt>, its destination sub-agent.
 *
 *					Upon the first match, the requestId is administered 
 *				in combination of its new Id, which is submitted to the 
 *				sub-agent, and the index (placeholder) of the variable in 
 *				the variablebindinglist. (indexlist)
 *
 * Author(s):   Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <snmpTypes.h> 
#include <UT-SNMP.h> 


#define AsnVbl VarBindList

typedef struct _INDREG {
    AsnInt type ;                                      /* spm identifier */
    AsnInt sid ;                                      /* spm identifier */
    AsnInt aid ;                              /* application identifier */
    AsnInt i ;                            /* variablebindingslist index */
	AsnInt view ;										  /* view index */
	AsnInt etype ;										  /* error type */
	AsnInt timestamp ;
	AsnVbl *vbl ;
    struct _INDREG *next ;
} INDREG ;

static INDREG *IndRegList = NULL ;
static AsnInt AID = 1234567890 ;
#define INDLISTTIMEOUT 300 



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	inRegister
 *
 * Arguments:	type, operation type (IN)
 *				sid, spm identifier (IN)
 *				aid, application identifier (IN/OUT)
 *				i, placeholder (IN)
 *				vi, view index (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function registers the triple <sid,aid,i>.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
inRegister( AsnInt type, AsnInt *sid, AsnInt *aid, AsnInt *i, AsnInt vi )
{
	INDREG *p = IndRegList ;
    struct timeval tp;

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR;

	p = (INDREG *) malloc( sizeof( INDREG ) ) ;

	p->sid = *sid ;
	p->aid = *aid = ++AID ;
	p->i = *i ;
	p->view = vi ;
	p->etype = NOERROR ;
	p->etype = type ;
    p->timestamp = tp.tv_sec ;
	p->vbl = NULL ;
	p->next = IndRegList ; IndRegList = p ;

	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	inGetView
 *
 * Arguments:	id, application identifier (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function retrieves the viewindex.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
inGetView( AsnInt id )
{
	INDREG *p = IndRegList ;

	while( p ) {
		if( p->aid == id ) return p->view ;
		p = p->next ;
	}
	return 0 ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	inUpdate
 *
 * Arguments:	aid, application identifier (IN)
 *				vbl, variablebindinglist (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function updates the indreglist with the returned
 *				variablebindinglist. Afterwards, it checkes whether or not
 *				the entire veriablebindinglist already is returned ?!
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
inUpdate( AsnInt *aid, AsnInt etype, AsnVbl *vbl )
{
	INDREG *p = IndRegList ;
    struct timeval tp;

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR;

	while( p ) {
		if( p->aid == *aid ) 
		{
			p->vbl = vbl ;
			p->timestamp = tp.tv_usec ;
			p->etype = etype ;
			return OK ;
		}
		p = p->next ;
	}
	p = IndRegList ;
	return ERROR ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	inAssembly
 *
 * Arguments:	aid, application identifier (IN)
 *				etype, errortype (OUT)
 *				eindex, errorindex (OUT)
 *				vbl, variablebindinglist (OUT)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function assembles the entire variablebindlist out
 *				of reponse-pieces of varbindlist found in the indreglist.
 *				It appends the sub varbindlists and removes them from the 
 *				indreglist.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
inAssembly( AsnInt type, 
			AsnInt *aid, AsnInt *etype, AsnInt *eindex, AsnVbl **vbl )
{
	INDREG *pp = NULL, *p = IndRegList ;
	AsnInt i = 0, sid = -1 ;

	*vbl = NULL;

    p = IndRegList ;
    while( p ) {
        if( p->aid == *aid ) { sid = p->sid ; break ; }
        p = p->next ;
    }
	if( sid == -1 ) return ERROR ;

    p = IndRegList ;
    while( p ) {
        if( (p->sid == sid) && (!(p->vbl) ) ) return ERROR ;
        p = p->next ;
    }

    p = IndRegList ;
	for( i = 0 ; i < 4 ; i++ ) 
	{
		for( pp = p = IndRegList; p != NULL; p = p->next )
		{
			if( (p->sid == sid) && (p->i == i) ) 
			{
				AsnListConcat( vbl, p->vbl ) ;
				if( p->etype != NOERROR )
				{
					*etype = p->etype ;
					*eindex = p->i ;
				}
				if( p == IndRegList )
					IndRegList = p->next ;
				else pp->next = p->next ; 
				free( p ) ;
			} else
				pp = p ;
		}
	}
	*aid = sid ;
	return( OK ) ;
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    inTimeout
 *
 * Arguments:   none
 *
 * Returns:     OK on success, otherwise ERROR
 *
 * Purpose:     This function removes expired indlist entries.
 *
 * Remarks:     none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
inTimeout( )
{
	INDREG *pp = NULL, *p = IndRegList ;
    struct timeval tp ;
	VarBind *vb;
	AsnInt type, etype;
	AsnOcts *a=pNewOct( 32 );

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR;
    p = IndRegList ;
    for( pp = p = IndRegList; p != NULL; p = p->next )
    {
        if( p->timestamp + INDLISTTIMEOUT < tp.tv_sec )
        {
			if( (p->type == UTB_GETIN_TYPE) && (p->vbl) )
			{
				FOR_EACH_LIST_ELMT( vb, p->vbl )
				{
					mrCheck( UTB_SETIN_TYPE, vb->name, NULL, NULL, a ); 
				}
				type = UTB_PERROR_TYPE ; etype = UT_TIMEOUT;
				upmSend( a->octs, &(p->aid), NULL, NULL, &etype, NULL, NULL,
						 NULL, NULL, NULL, NULL ) ;
			}
            if( p == IndRegList )
                IndRegList = p->next ;
            else pp->next = p->next ;
            free( p ) ;
        } else 
			pp = p ;
    }
	pFreeOct( a ) ;
	return OK ;
}


/* -----------------------------------------------------------------------
 */
