/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    simCL.c
 *
 * Date:        July 1994
 *
 * Purpose:		This file maps abstract data types upon real data types 
 *				and distributes active to the appropriate
 *				underlying interprocess communication domain.
 *
 * Author(s);   Alex Bos, John van Zunderd, 
 *				Vincent Berkhout, Frederic van Hengstum, Rob Post 
 *
 * -----------------------------------------------------------------------
 */
#include <string.h>
#include <sim.h>
#include <snmp_lDPI.h>

#define WAITING_TIME 3			 /* waiting time in usec */
#define messageLength 5420
#define PACKETSIZE      (16*1024)       /* 64 K minus 36 bytes */
extern int connection_fds[]; 
extern int connection_types[];
extern char dpi_buffer[]; 


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    CheckInputs
 *
 * Arguments:   *allHandles,	 in, all available handle (offsets)
 *				*activeHandles;	 in, all handles (offsets) that are active
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 *				This call checks whether data is ready to be received on the 
 *				given handles "allHandles". The handles for which data is 
 *				returned by "activeHandles".
 *
 * Remarks:		returns nrActiveHandles on success!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CheckInputs( int *allHandles, int *activeHandles ) 
{	
	int	i, j, nrActiveHandles=0;	
	struct 	timeval	time_out={0,10};

 	for (i = 0; i <= MAX_CONNECTIONS ; i++) 
 	{
	  if (allHandles[i] == -1)
	    break;

		if (wait_for_data(connection_fds[allHandles[i]], WAITING_TIME )==DPI_RC_OK)  /*1=timeout.tv_usec*/
		{
			activeHandles[i] = allHandles[i] ;
			nrActiveHandles++;
		}
	}	/* endfor i to maxFiles */

	return nrActiveHandles ;
}	/* end of function CheckInputs */


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfCheckInputs
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				allHandles, all handles that might be active (IN)
 *				activeHandles, all active handles (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfCheckInputs( IDOMAIN domain, IHANDLE *allHandles, IHANDLE *activeHandles ) 
{
	switch( domain )
	{
		case TCP_DOMAIN:
			return CheckInputs( (int *)allHandles, (int *)activeHandles ) ; 
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfClose
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				handle, handle to close (IN)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfClose( IDOMAIN domain, IHANDLE handle)
{
	switch( domain )
	{
		case TCP_DOMAIN:
			DPIdisconnect_from_agent(handle);
			return OK;
	}
	return FAILED ;
}


/* 
/*********************************************************************/
/* common routine to create a socket, bind it and then connect if it */
/* is a SOCK_STREAM socket.                                          */
/* It returns -1 if we fail;                                         */
/* returns handle(index) into connections array if success.          */
/* replacement of the Passiveopen				     */
/*********************************************************************/

int setup_passiveSock(int portNr )             		        /* setup/listen  socket  */  

{
   int                 i, connectSocket;                 		/* for loop control        */
   struct sockaddr_in  listenSock;           		/* source socket structure */
   unsigned long       host_addr;         		/* for IP address          */

   for (i=0; i < MAX_CONNECTIONS; i++)			/* all entries free    */
   {                                   			/* when we get here for    */
          connection_fds[i] = -1;         		/* the first time          */
   } 

   connectSocket = socket(PF_INET, SOCK_STREAM,0);  	/* create a socket         */
   if (connectSocket < 0) {                          	/* failure to create socket*/
      DPI_DEBUG(1,PERROR("socket():"));   		/* print error msg and     */
      return(-1);                         		/* -1 means failure        */
   } /* endif */
 
   memset(&listenSock,0,sizeof(listenSock));      	/* clear source structure  */
   listenSock.sin_family      = AF_INET;      		/* set AF_INET family      */
   listenSock.sin_port        = htons(portNr);		/* give us a port          */
   listenSock.sin_addr.s_addr =               		/* and we are hapy with    */
                       htonl(INADDR_ANY); 		/* any local IP address    */
   if( bind(connectSocket,                          	/* bind our socket (fd)    */
             (struct sockaddr *)&listenSock,  		/* to this address         */
             sizeof(listenSock)) < 0)            	/* length of the structure */
   {                         				/* if we failed, then      */
      DPI_DEBUG(1,PERROR("bind():"));     		/* print error msg and     */
      close(connectSocket);                         	/* close the socket, and   */
      return(-1);                         		/* -1 means failure        */
   } /* endif */
   if( listen(connectSocket,5) < 0 )
   {
     DPI_DEBUG(1,PERROR("listen():")); 
     close(connectSocket);
     return(ERROR);
   }
   connection_fds[0]=connectSocket;			
   return 0;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfPassiveOpen
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				localTAddress, local address containing the portnumber (IN)
 *				handle, a listen handle for new connections (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfPassiveOpen( IDOMAIN domain, IADDRESS localAddress, IHANDLE *handle ) 
{
	switch( domain )
	{
		case TCP_DOMAIN:
			if( ( *handle = setup_passiveSock( 8160 ) ) == 0 )
				return OK ;
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfActiveOpen
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				address, address of remote entity (IN)
 *				handle, the handle of the new connection (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfActiveOpen( IDOMAIN domain, IADDRESS l, IADDRESS r, IHANDLE *handle )  
{
	char hostname[128];

	switch( domain )
	{
		case TCP_DOMAIN:
			gethostname( hostname, 128 ); 
			if( ( *handle = setup_socket( domain, hostname, 8160 ) ) >= 0 )
				return OK ;
	}
	return FAILED ;
}


	STATUS
AcceptOpen( int *connectSocket, int *handle, u_char * remoteTAddress) 
{

  struct sockaddr_in	remote;
  int	remoteAddressLen, msgSize = PACKETSIZE, hostNr,i ;
  short portNr ;

  remoteAddressLen = sizeof( remote ) ;
  for (i = 1; i <= MAX_CONNECTIONS; i++) 
  {
    if (connection_fds[i]==-1)
      break;
  }
  if (i==MAX_CONNECTIONS)
    return FAILED;
  *handle=i;
  connection_fds[*handle] = accept( *connectSocket, (struct sockaddr *)&remote, 
						  &remoteAddressLen) ;

  if( connection_fds[*handle] < 0 ) 
  {
    connection_fds[*handle]=-1;
    return FAILED ;
  }
    if( setsockopt( connection_fds[*handle], SOL_SOCKET, SO_SNDBUF, 
				    (char *)&msgSize, sizeof(msgSize) ) < 0 ) 
      perror( "tcp" ) ;
    if( setsockopt( connection_fds[*handle], SOL_SOCKET, SO_RCVBUF, 
				    (char *)&msgSize, sizeof(msgSize) ) < 0 ) 
      perror( "tcp" ) ;
    connection_types[*handle]=TCP_connection;
    hostNr = ntohl(remote.sin_addr.s_addr) ;
    portNr = ntohs(remote.sin_port) ;
    memcpy( remoteTAddress, &hostNr, 4 ) ;
    memcpy( remoteTAddress+4, &portNr, 2) ;
	
    return( OK ) ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfAcceptOpen
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				connectHandle, the listen handle (IN)
 *				address, the address of the remote peer (OUT)
 *				handle, the handle referencing to new connection (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfAcceptOpen( IDOMAIN domain, IHANDLE *connectHandle, 
			   IADDRESS address, IHANDLE *handle )  
{
	switch( domain )
	{
		case TCP_DOMAIN:
			return( AcceptOpen( &connection_fds[(int)*connectHandle],
					   (int *)handle, (const char*) address) ); 
	}	
	return FAILED ;
}
 

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfSend
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				handle, data handle on which to send the msg (IN)
 *				message, the msg to be send (IN)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfSend( IDOMAIN domain, IHANDLE handle, IMESSAGE message )  
{
	switch( domain )
	{
		case TCP_DOMAIN:
			return DPIsend_packet_to_agent( handle, message, messageLength ) ;
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfReceive
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				handle, data handle on which to receive the msg (IN)
 *				msg, the msg that has been received (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfReceive( IDOMAIN domain, IHANDLE *handle, IMESSAGE msg )  
{	
	switch( domain )
	{
		case TCP_DOMAIN:
			if( read_dpi_packet_on_fd( connection_fds[*handle], 
									   TCP_connection ) > 0 )
			{
			  	memcpy( msg, dpi_buffer, messageLength ) ;
			  	return OK ;
			}
	}
	return FAILED ;
}
 

/* -----------------------------------------------------------------------
 */
