/* ----------------------------------------------------------------------- 
 *
 * Project:		UT-SNMP
 *
 * Filename:    simUPF.c 	(Snmp application Interface Module)
 *							(Snmp protocol machine interface Module)
 *
 * Date:    	July 1994
 *
 * Purpose:		This file implements the sim interface to provide the sim
 *				service to andimplements the upper protocol functions, as 
 *				well.
 *
 * Author(s): 	Alex Bos, John van Zunderd,
 *				Vincent Berkhout, Eric van de Hengst'm, Rob Post. 
 *
 * -----------------------------------------------------------------------
 */

#include <sim.h>

/*
 * -----------------------------------------------------------------------
 *
 *	GLOBAL TABLE DECLARATION
 *
 * -----------------------------------------------------------------------
 */ 
static ILISTENTABLE listenTable; 
static ICONNECTTABLE connectTable;


/*  
 * -----------------------------------------------------------------------
 *
 * Function:	simInit	  
 *
 * Arguments:	none
 *
 * Function:	OK, successfull initialization fo global data
 *				FAILED, second (or more) attempt to initialize data
 *				ERROR, no situations applicable.
 *
 * Purpose:		Initialize the glocal data, connectTable and listenTable,
 *				once!!
 *
 * Remark:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
simInit( )
{
	IDOMAIN i;
	short j;
    static BOOLEAN init = FALSE;

    if( init == FALSE ) {
    		for( j=0; j<ICONNECTMAX; j++)
    		{
    			connectTable[j].dataHandle=-1;
    			connectTable[j].dataDomain=-1;
    		}
    	    	for( j=0; j<MSGP_DOMAIN+1; j++)
   		{
    		   	listenTable[j].listenHandle=-1;
    		}
		init = TRUE;
		return( OK );
	}
	return( FAILED );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	simOpen
 *
 * Arguments:	l, symbolic address of local application (IN). 
 *				r, symbolic address of remote application (IN). 
 *
 * Return:		OK, successfull passive or active open.
 *				FAILED, no connection could be established in active open 
 *						situation.
 *				ERROR, initialization of global data failed,
 *					   no ipc domain could be opened in passive open situation.
 *
 * Purpose:		This function opens the underlying transport service. It 
 *				does so by performing the following algoritm. 
 * 				1) Initialize the local data, once, 
 *				2) open an listen endpoint for every known domain, OR
 *				3) open an connection to the remote application. Do this,
 *				   by attempting to connect to remote application through
 *				   an underlying ipc domain until an connection attempt 
 *				   succeeds. The domains are gradually ordered from 
 *				   reliable to unreliable.
 *
 * Remark:		When arguments r value is empty ("") indicates this an 
 *				passive open, and otherwise an active open !!!
 *
 * -----------------------------------------------------------------------
 */
STATUS simOpen( IADDRESS l, IADDRESS r ) 
{ 
	IHANDLE h, size; 
	STATUS status = ERROR;
 	short i; 
	IDOMAIN d; 
	char msg[5420];


	if( simInit() == ERROR )
		return( ERROR );

    if( strcmp( r, "" ) == 0 ) /* Passive Open */
	{
		/* open all known ipc domains */
		for( d = TCP_DOMAIN; d <= TCP_DOMAIN; d++ )
			if( lpfPassiveOpen( d, l, &listenTable[d].listenHandle ) == OK )
				status = OK;
	    return( status );
	} 
	else {	/* Active Open */

		for( i = 0; i < ICONNECTMAX; i++ )
			if( connectTable[i].dataHandle == (IHANDLE) -1 ) break;
		if( i == ICONNECTMAX ) return FAILED;

		for( d = TCP_DOMAIN; d <= TCP_DOMAIN; d++ )
    		if( lpfActiveOpen( d, l, r, &h ) == OK ) 
			{
				/* administrate new connection */
				memcpy( connectTable[i].remoteAddress, r, strlen( r ) );
				connectTable[i].dataHandle = h; 
				connectTable[i].dataDomain = d;

				/* submit an msg with name of local service user */
				memcpy( msg+4, l, size=strlen( l ) ); 
				memcpy( msg, &size, 4 );
				if( d == UDP_DOMAIN ) h = 54321;
				if( lpfSend( d, h, msg ) == OK )
					return( OK );
			
				/* on failure, close connection */
				simClose( connectTable[i].remoteAddress );
			}
	}
	return( FAILED );
}



/* 
 * ------------------------------------------------------------------------
 *
 * Function:	simSend	  
 *
 * Arguments:	a, address for remote application (IN).
 *				message, reference for message (IN).
 *
 * Return:		OK,	succesfull submitted message to remote application.
 *				FAILED, no connection to remote application available or
 *					    sending of message failed.
 *				ERROR, connection closed.
 *
 * Purpose:		This function submits an message to an remote application.
 *
 * Remarks:		Parameter a indicates the name of the closed connection !!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
simSend( IADDRESS a, IMESSAGE message )
{
    short i;
	int null = 0;
    IHANDLE dataHandle = DEFAULTVALUE;

    for( i = 0; i < ICONNECTMAX; i++ ) 
		if( strncmp( connectTable[i].remoteAddress, a, strlen(a) ) == 0 ) {
    		if( lpfSend( connectTable[i].dataDomain, 
						 connectTable[i].dataHandle, message ) != OK ) {
				(void) simClose( a );
				return( ERROR );
    		}
    		return( OK );
		}
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	simReceive	  
 *
 * Arguments:	a, address for remote application (OUT).
 *				message, reference for message (OUT).
 *
 * Return:		OK, new connection established or
 *				    new message received.
 *				FAILED, running out of resources or
 *						no messages received
 *				ERROR, connection closed.
 *
 * Purpose:		This function establishes new connections and receives 
 *				messages from remote aspplications.
 *
 * Remarks:		Parameter a contains the name of remote application, incase 
 *				an connection closes or opens, while the value of the message 
 *				parameters is empty !!!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
simReceive( IADDRESS a, IMESSAGE message )
{
	IHANDLE allHandles[ICONNECTMAX], activeHandles[ICONNECTMAX], h, i, j, null=0;
	IDOMAIN d;

	for( d = TCP_DOMAIN; d <= TCP_DOMAIN; d++ )	/* pending connect requests */
	{
		for( i=0; i<ICONNECTMAX; i++)
		{
			allHandles[i] = -1;
			activeHandles[i] = -1;	
		}
		allHandles[0] = listenTable[d].listenHandle;

		if( lpfCheckInputs( d, allHandles, activeHandles ) == 1 )
			if( lpfAcceptOpen( d, &listenTable[d].listenHandle, a, &h ) == OK )
				if( lpfReceive( d, &h, message ) == OK ) 
				{
		        	for( j = 0; j < ICONNECTMAX; j++ )
            			if( connectTable[j].dataHandle == (IHANDLE) -1 ) break;
        			if( j == ICONNECTMAX ) return FAILED;

					memcpy( &i, message, 4 );
                	memcpy( connectTable[j].remoteAddress, message+4, i );
					connectTable[j].remoteAddress[i] = '\0';
                	connectTable[j].dataHandle = h;
					connectTable[j].dataDomain = d;

					strcpy( a, connectTable[j].remoteAddress );
				}
	}

	for( d = TCP_DOMAIN; d <= TCP_DOMAIN; d++ )	/* data request pending */
	{
		for( i=0; i<ICONNECTMAX; i++)
		{
			allHandles[i] = -1;
			activeHandles[i] = -1;	
		}

    	for( i = 0, j = 0; i < ICONNECTMAX; i++ ) 
			if( connectTable[i].dataDomain == d ) 
	   			allHandles[j++] = connectTable[i].dataHandle;
		if( d == UDP_DOMAIN ) allHandles[0] = listenTable[d].listenHandle;

		if( lpfCheckInputs( d, allHandles, activeHandles ) > 0 )
			for( i = 0; (activeHandles[i] != -1)&&(i < ICONNECTMAX); i++ )
			{
				for( j = 0; j < ICONNECTMAX; j++ ) 
						if( connectTable[j].dataHandle == activeHandles[i] ) 
							strcpy( a, connectTable[j].remoteAddress );

				if( lpfReceive( d, &activeHandles[i], message ) != OK ) 
				{
					simClose( a );
					return( ERROR );
				}
				return( OK );
			}
	}
    return( FAILED );
}
	

/*  
 * -----------------------------------------------------------------------
 *
 * Function:	simClose
 *
 * Arguments: 	a, remote application (IN).
 *
 * Return:		OK, always.
 *				FAILED, none
 *				ERROR, none.
 *
 * Purpose:		This function physically and administratively closes all 
 *				ipc domains, in the passive role, and one connection with 
 *				an remote application, in the active role.
 *
 * Remarks:		When the parameter a contains an empty value the passive 
 *				close role must be performed by the function, in all other 
 *				situations the active role.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
simClose( IADDRESS a  )
{
    short i;
    IDOMAIN d;

    if( strcmp( a, "" ) == 0 ) 	/* passive close */
	{
        for( d = TCP_DOMAIN; d <= TCP_DOMAIN; d++ )
            if( listenTable[d].listenHandle != (IDOMAIN) -1 ) 
			{
				lpfClose( d, listenTable[d].listenHandle );
				listenTable[d].listenHandle = (IDOMAIN) -1;
			}
	} else 	/* active close */
	{
        for( i = 0; i <= ICONNECTMAX; i++ )
            if( strncmp( a, connectTable[i].remoteAddress, strlen(a) ) == 0 )
			{
				lpfClose( connectTable[i].dataDomain, connectTable[i].dataHandle );
				connectTable[i].dataHandle=-1;
				connectTable[i].dataDomain=-1;

			}
	} 
	return( OK );
}


/* ----------------------------------------------------------------------- */ 
