/*
\begin{verbatim}
*/

/*********************************************************************/
/*                                                                   */
/* SNMP-DPI API - SNMP Distributed Protocol Interface                */
/*                Application Programming Interface                  */
/*                                                                   */
/* Feb 03, 1994 - Version 0.13i                                      */
/*                                                                   */
/* Copyright    - (C) International Business Machines Corp. 1994     */
/*                                                                   */
/*   Permission to use, copy, modify, and distribute this software   */
/*   and its documentation for any lawful purpose and without fee is */
/*   hereby granted, provided that this notice be retained unaltered,*/
/*   and that the names of IBM and all other contributors shall not  */
/*   be used in advertising or publicity pertaining to distribution  */
/*   of the software without specific written prior permission.      */
/*   No contributor makes any representations about the suitability  */
/*   of this software for any purpose.  It is provided "as is"       */
/*   without express or implied warranty.                            */
/*                                                                   */
/*   IBM AND ALL OTHER CONTRIBUTORS DISCLAIM ALL WARRANTIES WITH     */
/*   REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF    */
/*   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, TITLE,    */
/*   AND NON-INFRINGEMENT.                                           */
/*                                                                   */
/*   IN NO EVENT SHALL IBM OR ANY OTHER CONTRIBUTOR BE LIABLE FOR    */
/*   ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN      */
/*   CONTRACT, TORT, OR OTHER FORM OF ACTION, ARISING OUT OF OR IN   */
/*   CONNECTION WITH, THE USE OR PERFORMANCE OF THIS SOFTWARE.       */
/*                                                                   */
/* snmp_qDPI.c - System independent code for query_DPI_port():       */
/*             - mkDPIqport_packet() to create an SNMP GET packet to */
/*               get the SNMP DPI port from an agent.                */
/*             - extract_DPI_port() to extract the DPI port number   */
/*               from an SNMP RESPONSE packet.                       */
/*             - These functions are called by the system specific   */
/*               function query_DPI_port() which may be different    */
/*               for different platforms.                            */
/*********************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <snmp_lDPI.h>

static unsigned char  packet_buf[144]; /* can hold SNMPv1 packet     */
static int            packet_len;      /* to track the packet length */

/*********************************************************************/
/* this macro is used by mkDPIqport_packet() function                */
/*********************************************************************/
#define PDU_COPY(a,b)                         /* copy data into PDU */\
   if ((packet_len + (b))<sizeof(packet_buf)) /* buffer, but only if*/\
   {                                          /* it fits in buffer  */\
      memcpy(&packet_buf[packet_len],(a),(b));/* copy the data      */\
      packet_len += (b);                      /* add new length     */\
   } else {                                   /* if it does not fit */\
      DPI_DEBUG(1,DPI_DM(                     /* debug_lvl >= 1     */\
        DPI_M_PDU_BUFFER_OVERFLOW));          /* print this message */\
      return((unsigned char *)0);             /* and return error   */\
   } /* endif */                              /* to caller          */

/*********************************************************************/
/* these 2 macros are used by extract_DPI_port() function            */
/*********************************************************************/
#define bytes_after_dpiPort_is(a)      /* macro to check how many   */\
  (memcmp(dpiPort,                     /* bytes are after the OID   */\
      &pdu_p[pdu_len-dpiPort_len-(a)], /* for the dpiPort           */\
      dpiPort_len) == 0)
#define integer_of_length(a)           /* macro to check if we have */\
  (pdu_p[pdu_len-2-a] == 0x02) &&      /* an integer of a bytes     */\
  (pdu_p[pdu_len-1-a] == (a))          /* at the end of the PDU     */

/*********************************************************************/
/* The following data are used to construct a simple SNMPv1 GET PDU  */
/* to get the SNMP DPI port (TCP or UDP instance) from an SNMP agent.*/
/*********************************************************************/
static unsigned char v1_hdr[] =
{  0x30                         /* sequence. SNMPv1 ASN.1 header     */
};                              /* insert length of remaining packet,*/
                                /* not including the hdr and the     */
                                /* length itself.                    */
static unsigned char version[] =
{  0x02, 0x01, 0x00,            /* integer, len=1, value=0(version-1)*/
   0x04                         /* string,  community name           */
};                              /* .. insert communit name length    */
                                /* .. insert community name          */
static unsigned char get_hdr[] =
{  0xa0,                        /* SNMP GET request, then reamining  */
   0x1c,                        /* length=0x1c if below OIDlen=0x0b  */
   0x02, 0x01, 0x01,            /* integer, len=1, request_id   = 1  */
   0x02, 0x01, 0x00,            /* integer, len=1, error_status = 0  */
   0x02, 0x01, 0x00,            /* integer, len=1, error_index  = 0  */
   0x30, 0x11,                  /* varbind list, len=0x11            */
   0x30, 0x0f                   /* varbind , len=0x0f                */
};
static unsigned char dpiPort[] =
{  0x06, 0x0b,                  /* object ID, len=0x0b               */
   0x2b, 0x06, 0x01, 0x04, 0x01,/* object ID itself (ASN.1 0x2b=1.3) */
   0x02, 0x02, 0x01, 0x01, 0x01,/* so it is: 1.3.6.1.4.1.2.2.1.1.1.0 */
   0x00                         /* instance 0                        */
};
static unsigned char nullValue[] =
{  0x05, 0x00                   /* NULL value, len = 0               */
};

/*********************************************************************/
/* Function to create an SNMP GET PDU to obtain the dpiPort variable */
/* (TCP or UDP) from an SNMP agent. This is system independent code. */
/* - If success, we return a ptr to a static buffer containing the   */
/*   the SNMP PDU. The callers pdu_len argument will be set to the   */
/*   length of that packet.                                          */
/* - If we fail, for whatever reason, a NULL ptr is returned and the */
/*   callers pdu_len argument will be set to zero.                   */
/*                                                                   */
/* Note(s):                                                          */
/* 1. Currently we only support SNMPv1 packets. But function proto   */
/*    type is ready to handle multiple versions.                     */
/* 2. Currently we only support two DPI port types:                  */
/*    - dpiPortForTCP (OID 1.3.6.1.4.1.2.2.1.1.1.0)                  */
/*    - dpiPortForUDP (OID 1.3.6.1.4.1.2.2.1.1.2.0)                  */
/* 3. For simplicity, we only handle ASN.1 BER length encoding for   */
/*    the short definite form, meaning 1 length byte, max length 127 */
/*    This limits the community length to some 90 octets, which      */
/*    seems a quite acceptable limit.                                */
/* 4. This function is not prototyped in the snmp_dpi.h include file */
/*    because it is not meant for DPI subagent programmers, but only */
/*    for those who implement the system specific query_DPI_port()   */
/*    function(s).                                                   */
/*********************************************************************/
unsigned char *mkDPIqport_packet(      /* Make a GET for DPIport     */
  int                    snmp_version, /* SNMP version (1=version-1) */
  int                    tcp_udp_port, /* 1=TCP or 2=UDP port        */
  char                  *community_p,  /* ptr to community name      */
  int                    community_len,/* length of community name   */
  int                   *pdu_len)      /* receives length of result  */
{
  unsigned char len;                   /* receives 1-byte length     */
  unsigned char *cp;                   /* ptr to PDU buffer          */

  cp = (unsigned char *)0;             /* init ptr to a NULL ptr     */
  packet_len = 0;                      /* init length of result      */
  *pdu_len   = 0;                      /* sofar we have no result    */

  if (snmp_version != 1) {             /* We only support SNMPv1 now */
     DPI_DEBUG(1,DPI_DM(               /* if debug_lvl >= 1 print    */
       DPI_M_INVALID_SNMP_VERSION));   /* this message               */
     return(cp);                       /* return NULL ptr            */
  } /* endif */

  if (tcp_udp_port > 1) {              /* We know 0=TCP and 1=UDP    */
     DPI_DEBUG(1,DPI_DM(               /* if debug_lvl >= 1 print    */
       DPI_M_INVALID_DPI_PORT));       /* this message               */
    return(cp);                        /* return NULL ptr            */
  } /* endif */

  len =                                /* calculate remaining length:*/
        sizeof(version)     +          /* .. length of version       */
        1                   +          /* .. 1 byte cummunity length */
        sizeof(get_hdr)     +          /* .. length of GET hdr       */
        sizeof(dpiPort)     +          /* .. length of dpiPort OID   */
        sizeof(nullValue);             /* .. length of NULL value    */
  if (len + community_len <= 127) {    /* .. len must fit in 7 bits  */
     len += community_len;             /* .. length community name   */
  } else {                             /* if it does not fit, then   */
     DPI_DEBUG(1,DPI_DM(               /* if debug_lvl >= 1 print    */
       DPI_M_COMMUNITY_TOO_LONG));     /* this message               */
     return(cp);                       /* return NULL ptr            */
  } /* endif */

  dpiPort[sizeof(dpiPort)-2] =         /* set proper OID instance    */
                         tcp_udp_port; /* for the port we want       */

  PDU_COPY(v1_hdr,sizeof(v1_hdr));     /* copy header data, then     */
  PDU_COPY(&len,1);                    /* copy remaining length      */
  PDU_COPY(version,sizeof(version));   /* copy version to packet     */
  len = community_len & 0xff;          /* isolate 1-byte length      */
  PDU_COPY(&len,1);                    /* copy it to the packet      */
  PDU_COPY(community_p,len);           /* copy community to packet   */
  PDU_COPY(get_hdr,sizeof(get_hdr));   /* copy GET request to packet */
  PDU_COPY(dpiPort,sizeof(dpiPort));   /* copy dpiPort OID to packet */
  PDU_COPY(nullValue,sizeof(nullValue));  /* copy nullValue too      */
  *pdu_len = packet_len;               /* set resulting PDU length   */
  return(packet_buf);                  /* return ptr to PDU buffer   */
}                                      /* which is a static buffer   */

/*********************************************************************/
/* Function to extract the DPI port from an SNMP RESPONSE PDU.       */
/* - If success, returns the DPI port number                         */
/* - If failure, returns -1                                          */
/*                                                                   */
/* Note(s):                                                          */
/* 1. Currently we only support SNMPv1 packets. We just assume that  */
/*    the packet is always SNMPv1. If needed, we should be able to   */
/*    determine the SNMP version from the packet data.               */
/* 2. We only do a very basic and simple check to see if this is a   */
/*    properly formatted PDU. We just assume that the OID is the     */
/*    one that was last requested by us, so we can use dpiPort octet */
/*    string to check if proper OID came back, and to ensure we      */
/*    pick up 1, 2 or 3 bytes for the port properly. The 3 bytes     */
/*    occurs when port number goes above 32767. In that case, the    */
/*    first byte should really be zero (BER needs first bit to be 0) */
/*    Possible cases of data at the end of the packet:               */
/*      dpiPort.05.00            - NULL value, probably SNMP error   */
/*      dpiPort.02.01.xx         - Integer Value, length is 1 byte   */
/*      dpiPort.02.02.xx.xx      - Integer Value, length is 2 bytes  */
/*      dpiPort.02.03.xx.xx.xx   - Integer Value, length is 3 bytes  */
/*    The integer value is always in network byte order.             */
/* 3. Some earlier implementations have returned an int for the port.*/
/*    However, an integer (int) may be only 2-bytes on some machines */
/*    and then -1 is a valid value. So therefor I have defined this  */
/*    function to return a long int, in which case a -1 is not a     */
/*    valid port number.                                             */
/* 4. This function is not prototyped in the snmp_dpi.h include file */
/*    because it is not meant for DPI subagent programmers, but only */
/*    for those who implement the system specific query_DPI_port()   */
/*    function(s).                                                   */
/*********************************************************************/
long int extract_DPI_port(             /* extract DPI port numbser    */
  char                  *pdu_p,        /* ptr to the PDU buffer      */
  int                    pdu_len)      /* length of the PDU data     */
{
  int               port;              /* to construct port number   */
  int dpiPort_len = sizeof(dpiPort);

  if (bytes_after_dpiPort_is(2)) {     /* 2 bytes after dpiPort OID  */
     return(-1);                       /* so probably NULL value or  */
  } /* endif */                        /* some other error           */

  if (bytes_after_dpiPort_is(3)) {     /* 3 bytes after dpiPort OID  */
     if (integer_of_length(1)) {       /* so if it is 1-byte integer */
        return(pdu_p[pdu_len-1]);      /* then return value          */
     } else return(-1);                /* otherwise return -1        */
  } /* endif */

  if (bytes_after_dpiPort_is(4)) {     /* 4 bytes after dpiPort OID  */
     if (integer_of_length(2)) {       /* so if it is 2-byte integer */
        port  = (pdu_p[pdu_len-2]<<8); /* pick up high order byte    */
        port += (pdu_p[pdu_len-1]);    /* add low order byte         */
        return(port);                  /* and return to caller       */
     } else return(-1);                /* otherwise return -1        */
  } /* endif */

  if (bytes_after_dpiPort_is(5)) {     /* 4 bytes after dpiPort OID  */
     if (integer_of_length(3)) {       /* so if it is 3-byte integer */
        port  = (pdu_p[pdu_len-3]<<16);/* should be zero really      */
        port += (pdu_p[pdu_len-2]<<8); /* pick up high order byte    */
        port += (pdu_p[pdu_len-1]);    /* add low order byte         */
        return(port);                  /* and return to caller       */
     } else return(-1);                /* otherwise return -1        */
  } /* endif */
  return(-1);                          /* just in case we get here   */
}

/*
\end{verbatim}
*/
