/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    upm.c	(upper Protocol functions Module)
 *
 * Date:        November 1994
 *
 * Purpose:     This file realizes the upm interface. The interface pro-
 *				vides the upm service to the ipm layer. The interface can
 *				be separated in two groups of functions.
 *
 *				General functions:
 *					upmOpen, upmInvoke and upmClose.
 *
 *				Service Primitive functions:
 *					upm<X>In and upm<X>Cf.
 *
 *				There relation is depicted below.
 *
 *													.....	
 *				 Open   Invoke   <X>Cf	<X>Rq 	<X>In	<X>Rs	Close
 *					^	^		 	^	|			^	|		^      UPM
 *				----|---|-----------|---|-----------|---|-------|---------
 *					|	|		 	|	v		 	|	v		|      IPM
 *									.   .............   .
 *									.....................
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <UT-SNMP.h> 
#include <lpm.h> 


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	upmOpen
 *
 * Arguments:	none 
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function opens and initializes the upm layer.	
 *
 * Remark:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
upmOpen( )
{
    if( TraceInit( ) != OK ) 
	{
        Error( "unable to open Trace module") ;
        return ERROR ;
    }
    if( (simInit( ) != OK) || (simOpen( "spm", "" ) != OK) ) 
	{
        Error( "unable to open SIM" ) ;
        return ERROR ;
    }
    return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	upmClose
 *
 * Arguments:	none 
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function closes the upm layer.	
 *
 * Remark:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
upmClose( )
{
    /* simClose( ) ; */
    return OK ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmSend	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function submits an message to an sub-application.
 *
 * Remark:		none.
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmSend( AsnOct *a, AsnInt *id, AsnInt *type, AsnOcts *taddress, 
	     AsnInt *qos, AsnInt *etype, AsnInt *eindex, AsnOid *lb, 
		 AsnOid *ub, AsnOcts *le, AsnOid *lt, AsnVbl *vbl )
{
	AsnInt h = 0 ;

	if( msgOpen( &h ) != OK ) return ERROR ;
	if( msgPut( &h, id, type, taddress, qos, etype, eindex, lb, ub, le,
			    lt, vbl ) != OK ) return ERROR ;
	if( simSend( a, (char *) h ) != OK )
	{
		Error( "msg(%d) send failed sub-application(%s)", *id, a ) ;
		return ERROR ;
	}
	return OK ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmInvoke
 *
 * Arguments:	none
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function receives ipc messages from sub-
 *				applications and dispatches the ReQuest - and ReSponse 
 *				type messages to the appropriate procedures.
 *
 * Remark:		init, already initialized.  	h, ipc message handle,
 *				id, message identifier, 		type, message type,
 *				qos, quality of service, 		etype, errortype,
 *				eindex, errorindex, 			vi, view index,
 *				taddress, transport address, 	le, localentity,
 *				ra, remote address, 			lb, low-bound subtree,
 *				ub, upper-bound subtree,		lt, localtime,
 *				vars, variable binding list.
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmInvoke( )
{
	static AsnInt init=0 ;
	static AsnOcts *taddress, *le, *ra ;
	static AsnOid *lb, *ub, *lt ;
 	static AsnVbl *vars ;
	static AsnInt h=0 ;
	AsnInt id=0, type=0, qos=0, etype=0, eindex=0, vi=0 ;
	AsnData	data ;
	VarBind *vb=NULL ;


	if( ! init )
	{
		taddress = pNewOct( 6 ) ; le = pNewOid( 32 ) ; ra = pNewOid( 32 ) ;
		lb = pNewOid( 32 ) ; ub = pNewOid( 32 ); lt = pNewOid( 9 ) ;
		msgOpen( &h ) ; 
 		init = 1 ;
	}

	if( yp1Timeout( ra, &id ) == OK )
	{
		yp1UnMap( ra, &id ) ;
        type = UTB_PERROR_TYPE ; etype = UT_TIMEOUT ;
        upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL,
                 NULL, NULL, NULL, NULL, NULL ) ;
	}
	if( inTimeout( ) == OK )
	{
	}

	if( simReceive( ra->octs, (char *) h ) != OK ) return ERROR ;

	ra->octetLen = strlen( ra->octs ) ;
	vars = pNewVbl( ) ;
	if( msgGet( &h, &id, &type, taddress, &qos, &etype, &eindex, lb, ub,
			    le, lt, vars ) != OK ) return ERROR ;

	if( yp1CheckOperation( ra, type ) != OK )
	{
        type = UTB_PERROR_TYPE ; etype = UT_ILLEGAL_OPERATION ;
        upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL,
                 NULL, NULL, NULL, NULL, NULL ) ;
		return( ERROR ) ;
	}

	data.type = type ;
	data.etype = etype ;
	data.eindex = eindex ;
    data.vbl = vars ;

	switch( type ) {

	case UTB_OPEN_TYPE : 
		if( yp1AppOpen( ra, qos ) != OK ) 
		{
			Error( "sub-application (%s) connection failed.", ra->octs ) ;
			simClose( ra->octs ) ;
			break ;
		}
		break ;

	case UTB_CLOSE_TYPE : 
		if( yp1AppClose( ra ) == OK ) 
		{
			simClose( ra->octs ) ;
			exit( -1 ) ;
		}
		break ;

	case UTB_MIBREGRQ_TYPE : 
		if( mrRegister( ra, id, le, lt, lb, ub ) != OK ) 
		{
			type = UTB_PERROR_TYPE ; etype = UT_MIB_REG_FAILED ;	
			upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL, NULL, NULL,
					 NULL, NULL, NULL ) ;
		} else {
			type = UTB_MIBREGCF_TYPE ;	
			upmSend( ra->octs, &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, 
				 	 NULL, NULL, NULL ) ;
		}
		break ;

	case UTB_MIBUNREGRQ_TYPE :
		if( mrUnRegister( ra, id ) != OK )
		{
			type = UTB_PERROR_TYPE ; etype = UT_MIB_UNREG_FAILED ;	
			upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL, 
					 NULL, NULL, NULL, NULL, NULL ) ;
		} else {
			type = UTB_MIBUNREGCF_TYPE ;	
			upmSend( ra->octs, &id, &type, NULL, NULL, NULL, NULL, NULL,
					 NULL, NULL, NULL, NULL ) ;
		}
		break ;

	case UTB_GETRQ_TYPE :
		yp1Map( ra, &id ) ;
		return ipmGetRq( id, taddress, qos, vi, le, lt, data ) ;

	case UTB_GETRS_TYPE :
		if( inUpdate( &id, etype, vars ) != OK ) break ;
		if( inAssembly( type, &id, &(data.etype), &(data.eindex),
						&(data.vbl) ) == OK )
			ipmGetRs( id, vi, le, lt, data ) ;
		break ;

	case UTB_GETNEXTRQ_TYPE :
		yp1Map( ra, &id ) ;
		return ipmGetnextRq( id, taddress, qos, vi, le, lt, data ) ;

	case UTB_GETNEXTRS_TYPE :
		vb = (VarBind *) vars->first->data ; 
		if( viCheck( vi=inGetView(id), &(vb->name) ) == OK )
		{
			if( inUpdate( &id, etype, vars ) != OK ) break ;
			if( inAssembly( type, &id, &(data.etype), &(data.eindex), 
						    &(data.vbl) ) == OK )
				return ipmGetnextRs( id, vi, le, lt, data ) ;
		} else {
        	if( mrCheck( type, vb->name, le, lt, taddress ) != OK )
			{
				vb->varBindChoice->choiceId = VARBINDCHOICE_ENDOFMIBVIEW ;
				inUpdate( &id, NOERROR, vars ) ;
				if( inAssembly( type, &id, &(data.etype), &(data.eindex), 
					    	    &(data.vbl) ) == OK )
					return ipmGetnextRs( id, vi, le, lt, data ) ;
			}
            upmSend( taddress->octs, &id, &type, NULL, NULL, NULL,
					 NULL, NULL, NULL, le, lt, vars ) ;
		}
		break ;

	case UTB_GETBULKRQ_TYPE :
		yp1Map( ra, &id ) ;
		return ipmGetbulkRq( id, taddress, qos, vi, le, lt, data ) ;

	case UTB_GETBULKRS_TYPE: break ;
		if( inUpdate( &id, etype, vars ) != OK ) break ;
		if( inAssembly( type, &id, &(data.etype), &(data.eindex),
						&(data.vbl) ) == OK )
		return ipmGetbulkRs( id, vi, le, lt, data ) ;

	case UTB_SETRQ_TYPE :
		yp1Map( ra, &id ) ;
		return ipmSetRq( id, taddress, qos, vi, le, lt, data ) ;

	case UTB_SETRS_TYPE :
		if( inUpdate( &id, etype, vars ) != OK ) break ;
		if( inAssembly( type, &id, &(data.etype), &(data.eindex),
						&(data.vbl) ) == OK )
		{
			if( ipmSetRs( id, vi, le, lt, data ) == OK ) 
			{
				/* Confirm/commit */ ;
			}
		}
		break ;

	case UTB_SETCOMMITRS_TYPE :
        break ;

	case UTB_SETUNDORS_TYPE :
		break;

	case UTB_INFORMREGRQ_TYPE :
		if( yp1InformReg( ra, taddress ) != OK )
		{
			type = UTB_PERROR_TYPE ; etype = UT_INFORM_REG_FAILED ;
			upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL, NULL, NULL,
                     NULL, NULL, NULL ) ;
		}
		break ;

	case UTB_INFORMUNREGRQ_TYPE :
		if( yp1InformUnReg( ra, taddress ) != OK )
		{
			type = UTB_PERROR_TYPE ; etype = UT_INFORM_UNREG_FAILED ;
			upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL, NULL, NULL,
                     NULL, NULL, NULL ) ;
		}
		break ;

	case UTB_INFORMRQ_TYPE :
		yp1Map( ra, &id ) ;
		return ipmInformRq( id, taddress, qos, vi, le, lt, data ) ;

	case UTB_TRAPREGRQ_TYPE : break ;
		if( yp1TrapReg( ra, taddress ) != OK )
		{
			type = UTB_PERROR_TYPE ; etype = UT_TRAP_REG_FAILED ;
			upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL, NULL, NULL,
                     NULL, NULL, NULL ) ;
		}
		break ;

	case UTB_TRAPUNREGRQ_TYPE : break ;
		if( yp1TrapUnReg( ra, taddress ) != OK )
		{
			type = UTB_PERROR_TYPE ; etype = UT_TRAP_UNREG_FAILED ;
			upmSend( ra->octs, &id, &type, NULL, NULL, &etype, NULL, NULL, NULL,
                     NULL, NULL, NULL ) ;
		}
		break ;

	case UTB_TRAPRQ_TYPE: break ;
		return ipmTrapRq( vi, le, lt, data ) ;

	case UTM_GETRQ_TYPE: break ;

	case UTM_GETNEXTRQ_TYPE: break ;

	case UTM_SETRQ_TYPE: break ;

	case UTM_COMMITRQ_TYPE: break ;

	}

	return OK ;
}

 


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmGetIn	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm getindication service 
 *				primitive, which provides this service to the ipm 
 *				service user.
 *
 *				Algoritm:
 * 				Foreach varbinding:
 *					Register varbindingsindexes an viewindex
 * 					Check varbinding against viewindex and subtrees
 * 					Whether submit varbinding to ut-agent 
 *					or unregister varbindingindexes.
 * 				Assemble sub-results, submit to recipient
 *
 * Remark:		none.
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmGetIn(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
	AsnOcts *a = pNewOct(32) ;
	AsnVbl *vbl;
	AsnInt aid, i = 0, type = UTB_GETIN_TYPE, s ; 


    FOR_EACH_LIST_ELMT( vb, data->vbl ) 
	{
		if( inRegister( type, id, &aid, &i, *vi ) != OK )
		{
			Error( "msg(%d) index registation failed.", *id ) ;
			return ERROR ;
		}

    	if( (s = viCheck( *vi, &(vb->name))) != OK )
			vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;
		else if( (s = mrCheck( type, vb->name, le, lt, a )) != OK )
			vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;

		*((int *) AsnListAppend( (vbl=pNewVbl( )) )) = (int) vb ;
		if( s == OK )
		{
			upmSend( a->octs, &aid, &type, NULL, NULL, NULL, NULL, NULL,
                     NULL, le, lt, vbl ) ;
			pFreeVbl( vbl ) ;
		} else
        	inUpdate( &aid, NOERROR, vbl ) ;

		++i; 
	}
	
    if( inAssembly( type, &aid, &(data->etype), &(data->eindex), 
					&(data->vbl) ) == OK ) 
		ipmGetRs( *id, *vi, le, lt, data ) ;

	pFreeOct( a ) ;
    return OK ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmGetCf	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, view index (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, just data (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm getconfimation service 
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmGetCf(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
	AsnOcts *a = pNewOct(32) ;
	AsnInt h = 0, type = UTB_GETCF_TYPE ; 

    if( yp1UnMap( a, id ) != OK ) 
    {
		Error( "msg(%d) resolving <sub_agent> failed", *id ) ;
        return( ERROR ) ;
    }

	if( upmSend( a->octs, id, &type, NULL, NULL, &(data->etype), 
				 &(data->eindex), NULL, NULL, le, lt, data->vbl ) != OK ) 
	{
    	pFreeOct( a ) ;
		return( ERROR ) ;
	}

    pFreeOct( a ) ;
    return( OK ) ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmGetnextIn	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm getnextindication service 
 *				primitive, which provides this service to the ipm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmGetnextIn(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
    AsnOcts *a = pNewOct(32) ;
    AsnVbl *vbl;
    AsnInt aid, i = 0, type = UTB_GETNEXTIN_TYPE, s ;

    FOR_EACH_LIST_ELMT( vb, data->vbl )
    {
        if( inRegister( type, id, &aid, &i, *vi ) != OK )
        {
            Error( "msg(%d) index registation failed.", *id ) ;
            return ERROR ;
        }

        if( (s = viCheck( *vi, &(vb->name))) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;
        else if( (s = mrCheck( type, vb->name, le, lt, a )) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;

        *((int *) AsnListAppend( (vbl=pNewVbl( )) )) = (int) vb ;
        if( s == OK )
        {
            upmSend( a->octs, &aid, &type, NULL, NULL, NULL, NULL, NULL,
                     NULL, le, lt, vbl ) ;
            pFreeVbl( vbl ) ;
        } else
            inUpdate( &aid, NOERROR, vbl ) ;

        ++i;
    }

    if( inAssembly( type, &aid, &(data->etype), &(data->eindex),
                    &(data->vbl) ) == OK )
        ipmGetnextRs( *id, *vi, le, lt, data ) ;

    pFreeOct( a ) ;
    return OK ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmGetnextCf	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm getnextconfimation
 *				service. 
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmGetnextCf(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
	AsnOcts *a = pNewOct(32) ;
	AsnInt h = 0, type = UTB_GETNEXTCF_TYPE ; 

    if( yp1UnMap( a, id ) != OK ) 
    {
		Error( 	"msg(%d) resolving <sub_agent> failed", *id ) ;
        return ERROR ;
    }
	if( upmSend( a->octs, id, &type, NULL, NULL, &(data->etype),
                 &(data->eindex), NULL, NULL, le, lt, data->vbl ) != OK )
        return ERROR ;	
    pFreeOct(a) ;
    return( OK ) ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmGetbulkIn	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm getbulkindication service 
 *				primitive, which provides this service to the ipm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmGetbulkIn(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
    AsnOcts *a = pNewOct(32) ;
	AsnVbl *vbl = pNewVbl( ) ;
    AsnInt maxR, aid, s, i = 0, h = 0, type = UTB_GETBULKIN_TYPE ;

	FOR_EACH_LIST_ELMT( vb, data->vbl )
    {
        if( inRegister( type, id, &aid, &i, *vi ) != OK )
        {
            Error( "msg(%d) index registation failed.", *id ) ;
            return ERROR ;
        }

        if( (s = viCheck( *vi, &(vb->name))) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;
        else if( (s = mrCheck( type, vb->name, le, lt, a )) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;

        *((int *) AsnListAppend( (vbl=pNewVbl( )) )) = (int) vb ;
        if( s == OK )
        {
			if( i < data->eindex ) maxR = 1 ; else maxR = data->etype ;
            upmSend( a->octs, &aid, &type, NULL, NULL, &maxR, NULL, NULL,
                     NULL, le, lt, vbl ) ;
            pFreeVbl( vbl ) ;
        } else
            inUpdate( &aid, NOERROR, vbl ) ;

        ++i;
    }

    if( inAssembly( type, &aid, &(data->etype), &(data->eindex),
                    &(data->vbl) ) == OK )
        ipmGetbulkRs( *id, *vi, le, lt, data ) ;

    pFreeOct( a ) ;
    return OK ;

}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmGetbulkCf	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm getbulkconfirmation
 *				service. 
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmGetbulkCf(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
	AsnOcts *a = pNewOct(32) ;
	AsnInt h = 0, type = UTB_GETBULKCF_TYPE ; 

    if( yp1UnMap( a, id ) != OK ) 
    {
		Error( 	"msg(%d) resolving <sub_agent> failed", *id ) ;
        return ERROR ;
    }
	if( upmSend( a->octs, id, &type, NULL, NULL, &(data->etype),
        		 &(data->eindex), NULL, NULL, le, lt, data->vbl ) != OK )
    pFreeOct(a) ;
    return( OK ) ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmSetIn	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm setindication service 
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmSetIn( 
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
    AsnOcts *a = pNewOct(32) ;
	AsnVbl *vbl = pNewVbl( ) ;
    AsnInt aid, s=0, i = 0, h = 0, type = UTB_SETIN_TYPE ;

    FOR_EACH_LIST_ELMT( vb, data->vbl )
    {
        if( inRegister( type, id, &aid, &i, *vi ) != OK )
        {
            Error( "msg(%d) index registation failed.", *id ) ;
            return ERROR ;
        }

        if( (s = viCheck( *vi, &(vb->name))) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;
        else if( (s = mrCheck( type, vb->name, le, lt, a )) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;

        *((int *) AsnListAppend( (vbl=pNewVbl( )) )) = (int) vb ;
        if( s == OK )
        {
            upmSend( a->octs, &aid, &type, NULL, NULL, NULL, NULL, NULL,
                     NULL, le, lt, vbl ) ;
            pFreeVbl( vbl ) ;
        } else
            inUpdate( &aid, NOERROR, vbl ) ;

        ++i;
    }

    if( inAssembly( type, &aid, &(data->etype), &(data->eindex),
                    &(data->vbl) ) == OK )
        ipmSetRs( *id, *vi, le, lt, data ) ;

    pFreeOct( a ) ;
    return OK ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmSetCf	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm setconfirmation service 
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmSetCf(
	AsnInt *id, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
	AsnOcts *a = pNewOct(32) ;
	AsnInt h = 0, type = UTB_SETCF_TYPE ; 

    if( yp1UnMap( a, id ) != OK ) 
    {
		Error( 	"msg(%d) resolving <sub_agent> failed", *id ) ;
        return ERROR ;
    }
	if( upmSend( a->octs, id, &type, NULL, NULL, &(data->etype),
        		 &(data->eindex), NULL, NULL, le, lt, data->vbl ) != OK )
    pFreeOct(a) ;
    return( OK ) ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmInformIn	
 *
 * Arguments:	id, request identifier (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm informindication service 
 *				primitive, which provides this service to the ipm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmInformIn(
	AsnInt *id, AsnOcts *taddress, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
    AsnOcts *a = pNewOct(32) ;
    AsnInt aid, i = 0, h = 0, type = UTB_INFORMIN_TYPE ;

    FOR_EACH_LIST_ELMT( vb, data->vbl )
    {
        if( viCheck( *vi, &(vb->name)) != OK )
            vb->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT ;
	}

    if( yp1InformCheck( a, taddress ) != OK )
    {
		Error( 	"msg(%d) resolving <sub_agent> failed", *id ) ;
        return ERROR ;
    }
	if( upmSend( a->octs, &aid, &type, NULL, NULL, NULL, NULL, NULL,
                 NULL, le, lt, data->vbl ) != OK ) return ERROR ;

	if( ipmInformRs( *id, *vi, le, lt, data ) != OK )
	{
        Error( "msg(%d) informresponse (%u.%u.%u.%u) failed.", *id,
			   	taddress->octs[0], taddress->octs[1], taddress->octs[2],
				taddress->octs[3] ) ;
        return ERROR ;
	}
	pFreeOct(a) ;
	return OK ;
}


/* ^L
 * -------------------------------------------------------------------
 *
 * Function:    upmInformCf
 *
 * Arguments:   id, request identifier (IN)
 *              vi, viewindex (IN)
 *              le, localentity (IN)
 *              lt, localtime (IN)
 *              data, errorindex, errorstatus, variables (IN)
 *
 * Returns:     OK on success, otherwise FAILED
 *
 * Purpose:     This function realizes the upm informconfirmation service
 *
 * Remark:      none
 *
 * -------------------------------------------------------------------
 */
    STATUS
upmInformCf(
    AsnInt *id,
    AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
    VarBind *vb ;
    AsnOcts *a = pNewOct(32) ;
    AsnInt h = 0, type = UTB_INFORMCF_TYPE ;

    if( yp1UnMap( a, id ) != OK )
    {
        Error(  "msg(%d) resolving <sub_agent> failed", *id ) ;
        return ERROR ;
    }
    if( upmSend( a->octs, id, &type, NULL, NULL, &(data->etype),
                 &(data->eindex), NULL, NULL, le, lt, data->vbl ) != OK )
    pFreeOct(a) ;
    return( OK ) ;
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:	upmTrapIn	
 *
 * Arguments:	taddress, transport address (IN)
 *				vi, viewindex (IN)
 *				le, localentity (IN)
 *				lt, localtime (IN)
 *				data, errorindex, errorstatus, variables (IN)
 *
 * Returns:		OK on success, otherwise FAILED
 *
 * Purpose:		This function realizes the upm trapindication service 
 *				primitive, which provides this service to the upm 
 *				service user.
 *
 * Remark:		none
 *
 * -------------------------------------------------------------------
 */
	STATUS
upmTrapIn(
	AsnOcts *taddress, 
	AsnInt *vi, AsnOcts *le, AsnOid *lt, AsnData *data )
{
	return OK ;
}


/* -------------------------------------------------------------------
 */
