/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    view.c	(helicopter view module)
 *
 * Date:        September 1994
 *
 * Purpose:     This file realizes the view interface, which consists of
 *				the functions viCheck and viResolve.
 *
 * Author(s):   Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */
#include <spmPartyMIB.h> 

#define viewMsb( x ) 	( x & 0x80 )		/* msb ?= 1 */
#define viewMask( x ) 	( x & 0x80000000 )	/* mask ?= 1 */
#define viewShift( x ) 	( x = x << 1 )		/* shift left 1 bit */

#define AsnVbl VarBindList


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viewGet
 *
 * Arguments:	ve, viewentry (IN)
 *				vs, viewsubtree (OUT)
 *				m, viewmask (OUT)
 *				vt, viewtype (OUT)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function retrieves the contence of the referenced 
 *				viewentry.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viewGet( AsnInt ve, AsnOid *vs, AsnInt *m , AsnInt *vt )
{
	static AsnOcts *vm ;
	static AsnInt init = 0 ;

	if( ! init ) {
		vm = pNewOct( 32 );
		init = 1 ;
	}
	memset( vs->octs, '\0', vs->octetLen ) ; vs->octetLen = 0 ;
	if( MibGetViewSubtree( ve, vs ) != OK ) return ERROR ;
	if( MibGetViewMask( ve, vm ) != OK ) return ERROR ;
	*m=0; if( vm->octetLen ) memcpy( m, vm->octs, vm->octetLen ) ;
	if( MibGetViewType( ve, vt ) != OK ) return ERROR ;
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viewSubOid
 *
 * Arguments:	octs, oid (IN)
 *				i, index of next suboid (OUT)
 *				s, size of current suboid (OUT)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function splits an oid in suboids.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viewSubOid( AsnOid *o, AsnInt *i, AsnInt *s )
{
	*s = 1 ;
	while( (viewMsb( o->octs[(*i)++] )) && (*i <= o->octetLen) ) (*s)++ ;

	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viewMatch
 *
 * Arguments:	m, viewmask (IN/OUT)
 *				s1, size of suboid 1 (IN)
 *				o1, suboid 1 (IN)
 *				s2, size of suboid 2 (IN)
 *				o2, suboid 2 (IN)
 *				r, result (OUT)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function matches two corresponding suboids.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viewMatch( AsnInt *m, AsnInt s1, AsnOct *o1, AsnInt s2, AsnOct *o2, AsnInt *r )
{
	AsnInt c1 = 0, c2 = 0 ;

	*r = 0 ;
	viewShift( *m );
	if( ! viewMask( *m ) ) return OK ;
	if( (*r = s1 - s2) != 0 ) return OK ;
	memcpy( &c1, o1, s1 ) ; memcpy( &c2, o2, s2 ) ;
	*r = memcmp( &c1, &c2, 4 ) ; 
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viewResult
 *
 * Arguments:	l, length of oid (IN)
 *				vs, viewsubtree (IN)
 *				vt, viewtype (IN)
 *				s, status (OUT)
 *
 * Returns:		OK on success, otherwise ERROR.
 *
 * Purpose:		This function determines whether an oid is included or
 *				excluded from an matching view.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viewResult( AsnOid *vs, AsnInt vt, AsnInt *s )
{
	static AsnOid *o ;
	static AsnInt init = 0 ;
	
	if( ! init ) { o = pNewOid( 64 ) ; init = 1 ; }
	if( *s == 9999 ) o->octetLen = 0 ;
	if( vs->octetLen < o->octetLen ) return ERROR ; 
	if( ( vs->octetLen == o->octetLen ) && ( oidcmp( o, vs ) >= 0 ) ) return ERROR ;
	memcpy( o->octs, vs->octs, o->octetLen = vs->octetLen ) ;
	if( vt == VIEWINCLUDED ) *s = OK ; else *s = ERROR ;
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viCheck
 *
 * Arguments:	vi, viewindex (IN)
 *				v, variable (IN)
 *
 * Returns:		OK on match, otherwise ERROR
 *
 * Purpose:		This function check the variable against the MIB view, 
 *				referenced by the viewindex, whether it is contained with-
 *				in the MIB view or not.
 *
 *				Criteria (RFC 1445, p. 29) :
 *
 *				o The variable object identifier comprises at least as many
 *				  sub-identifiers as does the subtree object identifier.
 *
 *				o Every variable sub-identifier matches the corresponding
 *				  subtree sub-identifier, whenever the corresponding bit
 *				  in the associated mask is non-zero.
 *
 *				o Whenever the variable object identifier belongs to none 
 *				  of the relevant subtrees, the variable is not in the MIB 
 *				  view.
 *
 *				o Whenever the variable object identifier matches exactly
 *				  one subtree object identifier, the variable is included/
 *				  excluded in the relevant MIB view.
 *
 *				o Whenever the variable object identifier matches more than
 *				  one subtree object identifier, the subtree comprising the
 *				  most sub-identifiers is significant, and secondly, the 
 *				  lexicographically ordering is of importance.
 *
 * Remark:		ve, view entries.	i, i-th viewentry.	r, compare result.
 *				vs, viewsubtree.	i1, i1-th suboid.	c1, i1-th suboid.
 *				vm, viewmask.		i2, i2-th suboid.	c2, i2-th suboid.
 *				vt, viewtype.		s1, suboid size.	s, status.
 *				m, mask.			s2, suboid size.	l, suboid length.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viCheck( AsnInt vi, AsnOid *v )
{
#define MAXVIEWENTRIES 10
	AsnInt 	ve[ MAXVIEWENTRIES ], s = 9999, vm = 0, vt = 0 ;
	static AsnInt init = 0;
	AsnInt 	i = 0, i1 = 0, i2 = 0, s1 = 0, s2 = 0, r = 0 ;
 	static 	AsnOid *vs ;

	if( ! init ) { vs = pNewOid( 64 ) ; vs->octetLen = 0 ; init = 1 ; }

	for( i = 0; i < MAXVIEWENTRIES; i++ ) ve[i] = 0 ;
	if( MibGetView( vi, ve ) != OK ) return ERROR ;

	for( i = 0; i < MAXVIEWENTRIES; i++ ) {
		if( ve[ i ] == 0 ) continue ;
		if( viewGet( ve[ i ], vs, &vm, &vt ) != OK ) return ERROR ;
	    while( (i1 < v->octetLen) && (i2 < vs->octetLen) && (r == 0) ) {
			viewSubOid( v,  &i1, &s1 ) ; viewSubOid( vs, &i2, &s2 ) ;
			viewMatch( &vm, s1, v->octs+i1-s1, s2, vs->octs+i2-s2, &r ) ;  
    	}
		if( (!r) && (i2 == vs->octetLen) )
			viewResult( vs, vt, &s ) ; i1 = 0 ; i2 = 0 ; r = 0 ;
	}
    return s ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viCheckAll
 *
 * Arguments:	vi, viewindex (IN)
 *				vbl, variablebinding list (IN)
 *
 * Returns:		OK on match, otherwise ERROR
 *
 * Purpose:		This function check the variable against the MIB view, 
 *				referenced by the viewindex, whether it is contained with-
 *				in the MIB view or not.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viCheckAll( AsnInt vi, AsnVbl *vbl )
{
	VarBind *vb ;

	FOR_EACH_LIST_ELMT( vb,  vbl )
	{
		if( viCheck( vi, (AsnOid *) &(vb->name) ) != OK )
			return ERROR ;
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	viewResolve
 *
 * Arguments:	vi, viewindex (OUT)
 *				v, variable (IN)
 *
 * Returns:		OK on success, otherwise ERROR
 *
 * Purpose:		This function searches for an viewindex, identying the 
 *				subtree, which covers the variable.
 *
 * Remark:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
viewResolve( AsnOid *v, AsnInt *vi )
{
    return OK ;
}



/* -----------------------------------------------------------------------
 */
